/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary Information about a StartTask running on a Compute Node.
  *
 */
class StartTaskInformation {
  /**
   * Create a StartTaskInformation.
   * @property {string} state The state of the StartTask on the Compute Node.
   * Possible values include: 'running', 'completed'
   * @property {date} startTime The time at which the StartTask started
   * running. This value is reset every time the Task is restarted or retried
   * (that is, this is the most recent time at which the StartTask started
   * running).
   * @property {date} [endTime] The time at which the StartTask stopped
   * running. This is the end time of the most recent run of the StartTask, if
   * that run has completed (even if that run failed and a retry is pending).
   * This element is not present if the StartTask is currently running.
   * @property {number} [exitCode] The exit code of the program specified on
   * the StartTask command line. This property is set only if the StartTask is
   * in the completed state. In general, the exit code for a process reflects
   * the specific convention implemented by the application developer for that
   * process. If you use the exit code value to make decisions in your code, be
   * sure that you know the exit code convention used by the application
   * process. However, if the Batch service terminates the StartTask (due to
   * timeout, or user termination via the API) you may see an operating
   * system-defined exit code.
   * @property {object} [containerInfo] Information about the container under
   * which the Task is executing. This property is set only if the Task runs in
   * a container context.
   * @property {string} [containerInfo.containerId]
   * @property {string} [containerInfo.state] This is the state of the
   * container according to the Docker service. It is equivalent to the status
   * field returned by "docker inspect".
   * @property {string} [containerInfo.error] This is the detailed error string
   * from the Docker service, if available. It is equivalent to the error field
   * returned by "docker inspect".
   * @property {object} [failureInfo] Information describing the Task failure,
   * if any. This property is set only if the Task is in the completed state
   * and encountered a failure.
   * @property {string} [failureInfo.category] Possible values include:
   * 'userError', 'serverError'
   * @property {string} [failureInfo.code]
   * @property {string} [failureInfo.message]
   * @property {array} [failureInfo.details]
   * @property {number} retryCount The number of times the Task has been
   * retried by the Batch service. Task application failures (non-zero exit
   * code) are retried, pre-processing errors (the Task could not be run) and
   * file upload errors are not retried. The Batch service will retry the Task
   * up to the limit specified by the constraints.
   * @property {date} [lastRetryTime] The most recent time at which a retry of
   * the Task started running. This element is present only if the Task was
   * retried (i.e. retryCount is nonzero). If present, this is typically the
   * same as startTime, but may be different if the Task has been restarted for
   * reasons other than retry; for example, if the Compute Node was rebooted
   * during a retry, then the startTime is updated but the lastRetryTime is
   * not.
   * @property {string} [result] The result of the Task execution. If the value
   * is 'failed', then the details of the failure can be found in the
   * failureInfo property. Possible values include: 'success', 'failure'
   */
  constructor() {
  }

  /**
   * Defines the metadata of StartTaskInformation
   *
   * @returns {object} metadata of StartTaskInformation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'StartTaskInformation',
      type: {
        name: 'Composite',
        className: 'StartTaskInformation',
        modelProperties: {
          state: {
            required: true,
            serializedName: 'state',
            type: {
              name: 'Enum',
              allowedValues: [ 'running', 'completed' ]
            }
          },
          startTime: {
            required: true,
            serializedName: 'startTime',
            type: {
              name: 'DateTime'
            }
          },
          endTime: {
            required: false,
            serializedName: 'endTime',
            type: {
              name: 'DateTime'
            }
          },
          exitCode: {
            required: false,
            serializedName: 'exitCode',
            type: {
              name: 'Number'
            }
          },
          containerInfo: {
            required: false,
            serializedName: 'containerInfo',
            type: {
              name: 'Composite',
              className: 'TaskContainerExecutionInformation'
            }
          },
          failureInfo: {
            required: false,
            serializedName: 'failureInfo',
            type: {
              name: 'Composite',
              className: 'TaskFailureInformation'
            }
          },
          retryCount: {
            required: true,
            serializedName: 'retryCount',
            type: {
              name: 'Number'
            }
          },
          lastRetryTime: {
            required: false,
            serializedName: 'lastRetryTime',
            type: {
              name: 'DateTime'
            }
          },
          result: {
            required: false,
            serializedName: 'result',
            type: {
              name: 'Enum',
              allowedValues: [ 'success', 'failure' ]
            }
          }
        }
      }
    };
  }
}

module.exports = StartTaskInformation;
