/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Azure File Share mounting configuration.
 *
 */
class AzureFileShareReference {
  /**
   * Create a AzureFileShareReference.
   * @member {string} accountName Account name. Name of the Azure storage
   * account.
   * @member {string} azureFileUrl Azure File URL. URL to access the Azure
   * File.
   * @member {object} credentials Credentials. Information about the Azure
   * storage credentials.
   * @member {string} [credentials.accountKey] Storage account key. One of
   * accountKey or accountKeySecretReference must be specified.
   * @member {object} [credentials.accountKeySecretReference] Information about
   * KeyVault secret storing the storage account key. One of accountKey or
   * accountKeySecretReference must be specified.
   * @member {object} [credentials.accountKeySecretReference.sourceVault] Fully
   * qualified resource indentifier of the Key Vault.
   * @member {string} [credentials.accountKeySecretReference.sourceVault.id]
   * The ID of the resource
   * @member {string} [credentials.accountKeySecretReference.secretUrl] The URL
   * referencing a secret in the Key Vault.
   * @member {string} relativeMountPath Relative mount path. The relative path
   * on the compute node where the Azure File share will be mounted. Note that
   * all cluster level file shares will be mounted under $AZ_BATCHAI_MOUNT_ROOT
   * location and all job level file shares will be mounted under
   * $AZ_BATCHAI_JOB_MOUNT_ROOT.
   * @member {string} [fileMode] File mode. File mode for files on the mounted
   * file share. Default value: 0777. Default value: '0777' .
   * @member {string} [directoryMode] Directory mode. File mode for directories
   * on the mounted file share. Default value: 0777. Default value: '0777' .
   */
  constructor() {
  }

  /**
   * Defines the metadata of AzureFileShareReference
   *
   * @returns {object} metadata of AzureFileShareReference
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AzureFileShareReference',
      type: {
        name: 'Composite',
        className: 'AzureFileShareReference',
        modelProperties: {
          accountName: {
            required: true,
            serializedName: 'accountName',
            type: {
              name: 'String'
            }
          },
          azureFileUrl: {
            required: true,
            serializedName: 'azureFileUrl',
            type: {
              name: 'String'
            }
          },
          credentials: {
            required: true,
            serializedName: 'credentials',
            type: {
              name: 'Composite',
              className: 'AzureStorageCredentialsInfo'
            }
          },
          relativeMountPath: {
            required: true,
            serializedName: 'relativeMountPath',
            type: {
              name: 'String'
            }
          },
          fileMode: {
            required: false,
            serializedName: 'fileMode',
            defaultValue: '0777',
            type: {
              name: 'String'
            }
          },
          directoryMode: {
            required: false,
            serializedName: 'directoryMode',
            defaultValue: '0777',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureFileShareReference;
