/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Defines an image
 *
 * @extends models['MediaObject']
 */
class ImageObject extends models['MediaObject'] {
  /**
   * Create a ImageObject.
   * @property {object} [thumbnail] The URL to a thumbnail of the image
   * @property {string} [imageInsightsToken] The token that you use in a
   * subsequent call to the Image Search API to get additional information
   * about the image. For information about using this token, see the
   * insightsToken query parameter.
   * @property {string} [imageId] Unique Id for the image
   * @property {string} [accentColor] A three-byte hexadecimal number that
   * represents the color that dominates the image. Use the color as the
   * temporary background in your client until the image is loaded.
   * @property {string} [visualWords] Visual representation of the image. Used
   * for getting more sizes
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ImageObject
   *
   * @returns {object} metadata of ImageObject
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ImageObject',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '_type',
          clientName: '_type'
        },
        uberParent: 'ResponseBase',
        className: 'ImageObject',
        modelProperties: {
          _type: {
            required: true,
            serializedName: '_type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          readLink: {
            required: false,
            readOnly: true,
            serializedName: 'readLink',
            type: {
              name: 'String'
            }
          },
          webSearchUrl: {
            required: false,
            readOnly: true,
            serializedName: 'webSearchUrl',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            readOnly: true,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          image: {
            required: false,
            readOnly: true,
            serializedName: 'image',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'ImageObject'
            }
          },
          description: {
            required: false,
            readOnly: true,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          alternateName: {
            required: false,
            readOnly: true,
            serializedName: 'alternateName',
            type: {
              name: 'String'
            }
          },
          bingId: {
            required: false,
            readOnly: true,
            serializedName: 'bingId',
            type: {
              name: 'String'
            }
          },
          thumbnailUrl: {
            required: false,
            readOnly: true,
            serializedName: 'thumbnailUrl',
            type: {
              name: 'String'
            }
          },
          provider: {
            required: false,
            readOnly: true,
            serializedName: 'provider',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ThingElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ResponseBase',
                    className: 'Thing'
                  }
              }
            }
          },
          text: {
            required: false,
            readOnly: true,
            serializedName: 'text',
            type: {
              name: 'String'
            }
          },
          contentUrl: {
            required: false,
            readOnly: true,
            serializedName: 'contentUrl',
            type: {
              name: 'String'
            }
          },
          hostPageUrl: {
            required: false,
            readOnly: true,
            serializedName: 'hostPageUrl',
            type: {
              name: 'String'
            }
          },
          contentSize: {
            required: false,
            readOnly: true,
            serializedName: 'contentSize',
            type: {
              name: 'String'
            }
          },
          encodingFormat: {
            required: false,
            readOnly: true,
            serializedName: 'encodingFormat',
            type: {
              name: 'String'
            }
          },
          hostPageDisplayUrl: {
            required: false,
            readOnly: true,
            serializedName: 'hostPageDisplayUrl',
            type: {
              name: 'String'
            }
          },
          width: {
            required: false,
            readOnly: true,
            serializedName: 'width',
            type: {
              name: 'Number'
            }
          },
          height: {
            required: false,
            readOnly: true,
            serializedName: 'height',
            type: {
              name: 'Number'
            }
          },
          thumbnail: {
            required: false,
            readOnly: true,
            serializedName: 'thumbnail',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'ImageObject'
            }
          },
          imageInsightsToken: {
            required: false,
            readOnly: true,
            serializedName: 'imageInsightsToken',
            type: {
              name: 'String'
            }
          },
          imageId: {
            required: false,
            readOnly: true,
            serializedName: 'imageId',
            type: {
              name: 'String'
            }
          },
          accentColor: {
            required: false,
            readOnly: true,
            serializedName: 'accentColor',
            type: {
              name: 'String'
            }
          },
          visualWords: {
            required: false,
            readOnly: true,
            serializedName: 'visualWords',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ImageObject;
