/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;

/**
 * Lists all the tasks for a specified container registry.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link TaskListResult} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _list(resourceGroupName, registryName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2019-04-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
    }
    if (registryName === null || registryName === undefined || typeof registryName.valueOf() !== 'string') {
      throw new Error('registryName cannot be null or undefined and it must be of type string.');
    }
    if (registryName !== null && registryName !== undefined) {
      if (registryName.length > 50)
      {
        throw new Error('"registryName" should satisfy the constraint - "MaxLength": 50');
      }
      if (registryName.length < 5)
      {
        throw new Error('"registryName" should satisfy the constraint - "MinLength": 5');
      }
      if (registryName.match(/^[a-zA-Z0-9]*$/) === null)
      {
        throw new Error('"registryName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]*$/');
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/tasks';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{registryName}', encodeURIComponent(registryName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['TaskListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Get the properties of a specified task.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Task} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _get(resourceGroupName, registryName, taskName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2019-04-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
    }
    if (registryName === null || registryName === undefined || typeof registryName.valueOf() !== 'string') {
      throw new Error('registryName cannot be null or undefined and it must be of type string.');
    }
    if (registryName !== null && registryName !== undefined) {
      if (registryName.length > 50)
      {
        throw new Error('"registryName" should satisfy the constraint - "MaxLength": 50');
      }
      if (registryName.length < 5)
      {
        throw new Error('"registryName" should satisfy the constraint - "MinLength": 5');
      }
      if (registryName.match(/^[a-zA-Z0-9]*$/) === null)
      {
        throw new Error('"registryName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]*$/');
      }
    }
    if (taskName === null || taskName === undefined || typeof taskName.valueOf() !== 'string') {
      throw new Error('taskName cannot be null or undefined and it must be of type string.');
    }
    if (taskName !== null && taskName !== undefined) {
      if (taskName.length > 50)
      {
        throw new Error('"taskName" should satisfy the constraint - "MaxLength": 50');
      }
      if (taskName.length < 5)
      {
        throw new Error('"taskName" should satisfy the constraint - "MinLength": 5');
      }
      if (taskName.match(/^[a-zA-Z0-9-_]*$/) === null)
      {
        throw new Error('"taskName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9-_]*$/');
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/tasks/{taskName}';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{registryName}', encodeURIComponent(registryName));
  requestUrl = requestUrl.replace('{taskName}', encodeURIComponent(taskName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}


/**
 * Creates a task for a container registry with the specified parameters.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} taskCreateParameters The parameters for creating a task.
 *
 * @param {object} [taskCreateParameters.identity] Identity for the resource.
 *
 * @param {string} [taskCreateParameters.identity.principalId] The principal ID
 * of resource identity.
 *
 * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
 * resource.
 *
 * @param {string} [taskCreateParameters.identity.type] The identity type.
 * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
 * UserAssigned', 'None'
 *
 * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
 * list of user identities associated with the resource. The user identity
 * dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
 * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {string} [taskCreateParameters.status] The current status of task.
 * Possible values include: 'Disabled', 'Enabled'
 *
 * @param {object} taskCreateParameters.platform The platform properties
 * against which the run has to happen.
 *
 * @param {string} taskCreateParameters.platform.os The operating system type
 * required for the run. Possible values include: 'Windows', 'Linux'
 *
 * @param {string} [taskCreateParameters.platform.architecture] The OS
 * architecture. Possible values include: 'amd64', 'x86', 'arm'
 *
 * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
 * Possible values include: 'v6', 'v7', 'v8'
 *
 * @param {object} [taskCreateParameters.agentConfiguration] The machine
 * configuration of the run agent.
 *
 * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
 * configuration in terms of number of cores required for the run.
 *
 * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
 *
 * @param {object} taskCreateParameters.step The properties of a task step.
 *
 * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
 * relative) of the source context for the task step.
 *
 * @param {string} [taskCreateParameters.step.contextAccessToken] The token
 * (git PAT or SAS token of storage account blob) associated with the context
 * for a step.
 *
 * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
 *
 * @param {object} [taskCreateParameters.trigger] The properties that describe
 * all triggers for the task.
 *
 * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
 * of timer triggers.
 *
 * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
 * of triggers based on source code repository.
 *
 * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
 * based on base image dependencies.
 *
 * @param {string}
 * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
 * of the auto trigger for base image dependency updates. Possible values
 * include: 'All', 'Runtime'
 *
 * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
 * current status of trigger. Possible values include: 'Disabled', 'Enabled'
 *
 * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
 * of the trigger.
 *
 * @param {object} [taskCreateParameters.credentials] The properties that
 * describes a set of credentials that will be used when this run is invoked.
 *
 * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
 * the credential parameters for accessing the source registry.
 *
 * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
 * The authentication mode which determines the source registry login scope.
 * The credentials for the source registry
 * will be generated using the given scope. These credentials will be used to
 * login to
 * the source registry during the run. Possible values include: 'None',
 * 'Default'
 *
 * @param {object} [taskCreateParameters.credentials.customRegistries]
 * Describes the credential parameters for accessing other custom registries.
 * The key
 * for the dictionary item will be the registry login server
 * (myregistry.azurecr.io) and
 * the value of the item will be the registry credentials for accessing the
 * registry.
 *
 * @param {string} taskCreateParameters.location The location of the resource.
 * This cannot be changed after the resource is created.
 *
 * @param {object} [taskCreateParameters.tags] The tags of the resource.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Task} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _create(resourceGroupName, registryName, taskName, taskCreateParameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginCreate(resourceGroupName, registryName, taskName, taskCreateParameters, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Deletes a specified task.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deleteMethod(resourceGroupName, registryName, taskName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginDeleteMethod(resourceGroupName, registryName, taskName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Updates a task with the specified parameters.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} taskUpdateParameters The parameters for updating a task.
 *
 * @param {object} [taskUpdateParameters.identity] Identity for the resource.
 *
 * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
 * of resource identity.
 *
 * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
 * resource.
 *
 * @param {string} [taskUpdateParameters.identity.type] The identity type.
 * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
 * UserAssigned', 'None'
 *
 * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
 * list of user identities associated with the resource. The user identity
 * dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
 * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {string} [taskUpdateParameters.status] The current status of task.
 * Possible values include: 'Disabled', 'Enabled'
 *
 * @param {object} [taskUpdateParameters.platform] The platform properties
 * against which the run has to happen.
 *
 * @param {string} [taskUpdateParameters.platform.os] The operating system type
 * required for the run. Possible values include: 'Windows', 'Linux'
 *
 * @param {string} [taskUpdateParameters.platform.architecture] The OS
 * architecture. Possible values include: 'amd64', 'x86', 'arm'
 *
 * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
 * Possible values include: 'v6', 'v7', 'v8'
 *
 * @param {object} [taskUpdateParameters.agentConfiguration] The machine
 * configuration of the run agent.
 *
 * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
 * configuration in terms of number of cores required for the run.
 *
 * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
 *
 * @param {object} [taskUpdateParameters.step] The properties for updating a
 * task step.
 *
 * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
 * relative) of the source context for the task step.
 *
 * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
 * (git PAT or SAS token of storage account blob) associated with the context
 * for a step.
 *
 * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
 *
 * @param {object} [taskUpdateParameters.trigger] The properties for updating
 * trigger properties.
 *
 * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
 * of timer triggers.
 *
 * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
 * of triggers based on source code repository.
 *
 * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
 * based on base image dependencies.
 *
 * @param {string}
 * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
 * type of the auto trigger for base image dependency updates. Possible values
 * include: 'All', 'Runtime'
 *
 * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
 * current status of trigger. Possible values include: 'Disabled', 'Enabled'
 *
 * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
 * of the trigger.
 *
 * @param {object} [taskUpdateParameters.credentials] The parameters that
 * describes a set of credentials that will be used when this run is invoked.
 *
 * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
 * the credential parameters for accessing the source registry.
 *
 * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
 * The authentication mode which determines the source registry login scope.
 * The credentials for the source registry
 * will be generated using the given scope. These credentials will be used to
 * login to
 * the source registry during the run. Possible values include: 'None',
 * 'Default'
 *
 * @param {object} [taskUpdateParameters.credentials.customRegistries]
 * Describes the credential parameters for accessing other custom registries.
 * The key
 * for the dictionary item will be the registry login server
 * (myregistry.azurecr.io) and
 * the value of the item will be the registry credentials for accessing the
 * registry.
 *
 * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Task} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _update(resourceGroupName, registryName, taskName, taskUpdateParameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginUpdate(resourceGroupName, registryName, taskName, taskUpdateParameters, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Returns a task with extended information that includes all secrets.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Task} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getDetails(resourceGroupName, registryName, taskName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2019-04-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
    }
    if (registryName === null || registryName === undefined || typeof registryName.valueOf() !== 'string') {
      throw new Error('registryName cannot be null or undefined and it must be of type string.');
    }
    if (registryName !== null && registryName !== undefined) {
      if (registryName.length > 50)
      {
        throw new Error('"registryName" should satisfy the constraint - "MaxLength": 50');
      }
      if (registryName.length < 5)
      {
        throw new Error('"registryName" should satisfy the constraint - "MinLength": 5');
      }
      if (registryName.match(/^[a-zA-Z0-9]*$/) === null)
      {
        throw new Error('"registryName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]*$/');
      }
    }
    if (taskName === null || taskName === undefined || typeof taskName.valueOf() !== 'string') {
      throw new Error('taskName cannot be null or undefined and it must be of type string.');
    }
    if (taskName !== null && taskName !== undefined) {
      if (taskName.length > 50)
      {
        throw new Error('"taskName" should satisfy the constraint - "MaxLength": 50');
      }
      if (taskName.length < 5)
      {
        throw new Error('"taskName" should satisfy the constraint - "MinLength": 5');
      }
      if (taskName.match(/^[a-zA-Z0-9-_]*$/) === null)
      {
        throw new Error('"taskName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9-_]*$/');
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/tasks/{taskName}/listDetails';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{registryName}', encodeURIComponent(registryName));
  requestUrl = requestUrl.replace('{taskName}', encodeURIComponent(taskName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Creates a task for a container registry with the specified parameters.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} taskCreateParameters The parameters for creating a task.
 *
 * @param {object} [taskCreateParameters.identity] Identity for the resource.
 *
 * @param {string} [taskCreateParameters.identity.principalId] The principal ID
 * of resource identity.
 *
 * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
 * resource.
 *
 * @param {string} [taskCreateParameters.identity.type] The identity type.
 * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
 * UserAssigned', 'None'
 *
 * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
 * list of user identities associated with the resource. The user identity
 * dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
 * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {string} [taskCreateParameters.status] The current status of task.
 * Possible values include: 'Disabled', 'Enabled'
 *
 * @param {object} taskCreateParameters.platform The platform properties
 * against which the run has to happen.
 *
 * @param {string} taskCreateParameters.platform.os The operating system type
 * required for the run. Possible values include: 'Windows', 'Linux'
 *
 * @param {string} [taskCreateParameters.platform.architecture] The OS
 * architecture. Possible values include: 'amd64', 'x86', 'arm'
 *
 * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
 * Possible values include: 'v6', 'v7', 'v8'
 *
 * @param {object} [taskCreateParameters.agentConfiguration] The machine
 * configuration of the run agent.
 *
 * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
 * configuration in terms of number of cores required for the run.
 *
 * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
 *
 * @param {object} taskCreateParameters.step The properties of a task step.
 *
 * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
 * relative) of the source context for the task step.
 *
 * @param {string} [taskCreateParameters.step.contextAccessToken] The token
 * (git PAT or SAS token of storage account blob) associated with the context
 * for a step.
 *
 * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
 *
 * @param {object} [taskCreateParameters.trigger] The properties that describe
 * all triggers for the task.
 *
 * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
 * of timer triggers.
 *
 * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
 * of triggers based on source code repository.
 *
 * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
 * based on base image dependencies.
 *
 * @param {string}
 * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
 * of the auto trigger for base image dependency updates. Possible values
 * include: 'All', 'Runtime'
 *
 * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
 * current status of trigger. Possible values include: 'Disabled', 'Enabled'
 *
 * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
 * of the trigger.
 *
 * @param {object} [taskCreateParameters.credentials] The properties that
 * describes a set of credentials that will be used when this run is invoked.
 *
 * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
 * the credential parameters for accessing the source registry.
 *
 * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
 * The authentication mode which determines the source registry login scope.
 * The credentials for the source registry
 * will be generated using the given scope. These credentials will be used to
 * login to
 * the source registry during the run. Possible values include: 'None',
 * 'Default'
 *
 * @param {object} [taskCreateParameters.credentials.customRegistries]
 * Describes the credential parameters for accessing other custom registries.
 * The key
 * for the dictionary item will be the registry login server
 * (myregistry.azurecr.io) and
 * the value of the item will be the registry credentials for accessing the
 * registry.
 *
 * @param {string} taskCreateParameters.location The location of the resource.
 * This cannot be changed after the resource is created.
 *
 * @param {object} [taskCreateParameters.tags] The tags of the resource.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Task} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginCreate(resourceGroupName, registryName, taskName, taskCreateParameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2019-04-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
    }
    if (registryName === null || registryName === undefined || typeof registryName.valueOf() !== 'string') {
      throw new Error('registryName cannot be null or undefined and it must be of type string.');
    }
    if (registryName !== null && registryName !== undefined) {
      if (registryName.length > 50)
      {
        throw new Error('"registryName" should satisfy the constraint - "MaxLength": 50');
      }
      if (registryName.length < 5)
      {
        throw new Error('"registryName" should satisfy the constraint - "MinLength": 5');
      }
      if (registryName.match(/^[a-zA-Z0-9]*$/) === null)
      {
        throw new Error('"registryName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]*$/');
      }
    }
    if (taskName === null || taskName === undefined || typeof taskName.valueOf() !== 'string') {
      throw new Error('taskName cannot be null or undefined and it must be of type string.');
    }
    if (taskName !== null && taskName !== undefined) {
      if (taskName.length > 50)
      {
        throw new Error('"taskName" should satisfy the constraint - "MaxLength": 50');
      }
      if (taskName.length < 5)
      {
        throw new Error('"taskName" should satisfy the constraint - "MinLength": 5');
      }
      if (taskName.match(/^[a-zA-Z0-9-_]*$/) === null)
      {
        throw new Error('"taskName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9-_]*$/');
      }
    }
    if (taskCreateParameters === null || taskCreateParameters === undefined) {
      throw new Error('taskCreateParameters cannot be null or undefined.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/tasks/{taskName}';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{registryName}', encodeURIComponent(registryName));
  requestUrl = requestUrl.replace('{taskName}', encodeURIComponent(taskName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PUT';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (taskCreateParameters !== null && taskCreateParameters !== undefined) {
      let requestModelMapper = new client.models['Task']().mapper();
      requestModel = client.serialize(requestModelMapper, taskCreateParameters, 'taskCreateParameters');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(taskCreateParameters, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError1 = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError1.request = msRest.stripRequest(httpRequest);
        deserializationError1.response = msRest.stripResponse(response);
        return callback(deserializationError1);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deletes a specified task.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginDeleteMethod(resourceGroupName, registryName, taskName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2019-04-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
    }
    if (registryName === null || registryName === undefined || typeof registryName.valueOf() !== 'string') {
      throw new Error('registryName cannot be null or undefined and it must be of type string.');
    }
    if (registryName !== null && registryName !== undefined) {
      if (registryName.length > 50)
      {
        throw new Error('"registryName" should satisfy the constraint - "MaxLength": 50');
      }
      if (registryName.length < 5)
      {
        throw new Error('"registryName" should satisfy the constraint - "MinLength": 5');
      }
      if (registryName.match(/^[a-zA-Z0-9]*$/) === null)
      {
        throw new Error('"registryName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]*$/');
      }
    }
    if (taskName === null || taskName === undefined || typeof taskName.valueOf() !== 'string') {
      throw new Error('taskName cannot be null or undefined and it must be of type string.');
    }
    if (taskName !== null && taskName !== undefined) {
      if (taskName.length > 50)
      {
        throw new Error('"taskName" should satisfy the constraint - "MaxLength": 50');
      }
      if (taskName.length < 5)
      {
        throw new Error('"taskName" should satisfy the constraint - "MinLength": 5');
      }
      if (taskName.match(/^[a-zA-Z0-9-_]*$/) === null)
      {
        throw new Error('"taskName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9-_]*$/');
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/tasks/{taskName}';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{registryName}', encodeURIComponent(registryName));
  requestUrl = requestUrl.replace('{taskName}', encodeURIComponent(taskName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'DELETE';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202 && statusCode !== 204) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Updates a task with the specified parameters.
 *
 * @param {string} resourceGroupName The name of the resource group to which
 * the container registry belongs.
 *
 * @param {string} registryName The name of the container registry.
 *
 * @param {string} taskName The name of the container registry task.
 *
 * @param {object} taskUpdateParameters The parameters for updating a task.
 *
 * @param {object} [taskUpdateParameters.identity] Identity for the resource.
 *
 * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
 * of resource identity.
 *
 * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
 * resource.
 *
 * @param {string} [taskUpdateParameters.identity.type] The identity type.
 * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
 * UserAssigned', 'None'
 *
 * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
 * list of user identities associated with the resource. The user identity
 * dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
 * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {string} [taskUpdateParameters.status] The current status of task.
 * Possible values include: 'Disabled', 'Enabled'
 *
 * @param {object} [taskUpdateParameters.platform] The platform properties
 * against which the run has to happen.
 *
 * @param {string} [taskUpdateParameters.platform.os] The operating system type
 * required for the run. Possible values include: 'Windows', 'Linux'
 *
 * @param {string} [taskUpdateParameters.platform.architecture] The OS
 * architecture. Possible values include: 'amd64', 'x86', 'arm'
 *
 * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
 * Possible values include: 'v6', 'v7', 'v8'
 *
 * @param {object} [taskUpdateParameters.agentConfiguration] The machine
 * configuration of the run agent.
 *
 * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
 * configuration in terms of number of cores required for the run.
 *
 * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
 *
 * @param {object} [taskUpdateParameters.step] The properties for updating a
 * task step.
 *
 * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
 * relative) of the source context for the task step.
 *
 * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
 * (git PAT or SAS token of storage account blob) associated with the context
 * for a step.
 *
 * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
 *
 * @param {object} [taskUpdateParameters.trigger] The properties for updating
 * trigger properties.
 *
 * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
 * of timer triggers.
 *
 * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
 * of triggers based on source code repository.
 *
 * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
 * based on base image dependencies.
 *
 * @param {string}
 * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
 * type of the auto trigger for base image dependency updates. Possible values
 * include: 'All', 'Runtime'
 *
 * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
 * current status of trigger. Possible values include: 'Disabled', 'Enabled'
 *
 * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
 * of the trigger.
 *
 * @param {object} [taskUpdateParameters.credentials] The parameters that
 * describes a set of credentials that will be used when this run is invoked.
 *
 * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
 * the credential parameters for accessing the source registry.
 *
 * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
 * The authentication mode which determines the source registry login scope.
 * The credentials for the source registry
 * will be generated using the given scope. These credentials will be used to
 * login to
 * the source registry during the run. Possible values include: 'None',
 * 'Default'
 *
 * @param {object} [taskUpdateParameters.credentials.customRegistries]
 * Describes the credential parameters for accessing other custom registries.
 * The key
 * for the dictionary item will be the registry login server
 * (myregistry.azurecr.io) and
 * the value of the item will be the registry credentials for accessing the
 * registry.
 *
 * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Task} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginUpdate(resourceGroupName, registryName, taskName, taskUpdateParameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2019-04-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (resourceGroupName !== null && resourceGroupName !== undefined) {
      if (resourceGroupName.length < 1)
      {
        throw new Error('"resourceGroupName" should satisfy the constraint - "MinLength": 1');
      }
    }
    if (registryName === null || registryName === undefined || typeof registryName.valueOf() !== 'string') {
      throw new Error('registryName cannot be null or undefined and it must be of type string.');
    }
    if (registryName !== null && registryName !== undefined) {
      if (registryName.length > 50)
      {
        throw new Error('"registryName" should satisfy the constraint - "MaxLength": 50');
      }
      if (registryName.length < 5)
      {
        throw new Error('"registryName" should satisfy the constraint - "MinLength": 5');
      }
      if (registryName.match(/^[a-zA-Z0-9]*$/) === null)
      {
        throw new Error('"registryName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9]*$/');
      }
    }
    if (taskName === null || taskName === undefined || typeof taskName.valueOf() !== 'string') {
      throw new Error('taskName cannot be null or undefined and it must be of type string.');
    }
    if (taskName !== null && taskName !== undefined) {
      if (taskName.length > 50)
      {
        throw new Error('"taskName" should satisfy the constraint - "MaxLength": 50');
      }
      if (taskName.length < 5)
      {
        throw new Error('"taskName" should satisfy the constraint - "MinLength": 5');
      }
      if (taskName.match(/^[a-zA-Z0-9-_]*$/) === null)
      {
        throw new Error('"taskName" should satisfy the constraint - "Pattern": /^[a-zA-Z0-9-_]*$/');
      }
    }
    if (taskUpdateParameters === null || taskUpdateParameters === undefined) {
      throw new Error('taskUpdateParameters cannot be null or undefined.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/tasks/{taskName}';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{registryName}', encodeURIComponent(registryName));
  requestUrl = requestUrl.replace('{taskName}', encodeURIComponent(taskName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PATCH';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (taskUpdateParameters !== null && taskUpdateParameters !== undefined) {
      let requestModelMapper = new client.models['TaskUpdateParameters']().mapper();
      requestModel = client.serialize(requestModelMapper, taskUpdateParameters, 'taskUpdateParameters');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(taskUpdateParameters, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Task']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError1 = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError1.request = msRest.stripRequest(httpRequest);
        deserializationError1.response = msRest.stripResponse(response);
        return callback(deserializationError1);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Lists all the tasks for a specified container registry.
 *
 * @param {string} nextPageLink The NextLink from the previous successful call
 * to List operation.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link TaskListResult} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listNext(nextPageLink, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (nextPageLink === null || nextPageLink === undefined || typeof nextPageLink.valueOf() !== 'string') {
      throw new Error('nextPageLink cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let requestUrl = '{nextLink}';
  requestUrl = requestUrl.replace('{nextLink}', nextPageLink);

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['TaskListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a Tasks. */
class Tasks {
  /**
   * Create a Tasks.
   * @param {ContainerRegistryManagementClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._list = _list;
    this._get = _get;
    this._create = _create;
    this._deleteMethod = _deleteMethod;
    this._update = _update;
    this._getDetails = _getDetails;
    this._beginCreate = _beginCreate;
    this._beginDeleteMethod = _beginDeleteMethod;
    this._beginUpdate = _beginUpdate;
    this._listNext = _listNext;
  }

  /**
   * Lists all the tasks for a specified container registry.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<TaskListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listWithHttpOperationResponse(resourceGroupName, registryName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._list(resourceGroupName, registryName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Lists all the tasks for a specified container registry.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {TaskListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link TaskListResult} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  list(resourceGroupName, registryName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._list(resourceGroupName, registryName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._list(resourceGroupName, registryName, options, optionalCallback);
    }
  }

  /**
   * Get the properties of a specified task.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getWithHttpOperationResponse(resourceGroupName, registryName, taskName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._get(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Get the properties of a specified task.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Task} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Task} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  get(resourceGroupName, registryName, taskName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._get(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._get(resourceGroupName, registryName, taskName, options, optionalCallback);
    }
  }

  /**
   * Creates a task for a container registry with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskCreateParameters The parameters for creating a task.
   *
   * @param {object} [taskCreateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskCreateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskCreateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskCreateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} taskCreateParameters.platform The platform properties
   * against which the run has to happen.
   *
   * @param {string} taskCreateParameters.platform.os The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskCreateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskCreateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} taskCreateParameters.step The properties of a task step.
   *
   * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskCreateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskCreateParameters.trigger] The properties that describe
   * all triggers for the task.
   *
   * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
   * of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskCreateParameters.credentials] The properties that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskCreateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {string} taskCreateParameters.location The location of the resource.
   * This cannot be changed after the resource is created.
   *
   * @param {object} [taskCreateParameters.tags] The tags of the resource.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createWithHttpOperationResponse(resourceGroupName, registryName, taskName, taskCreateParameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._create(resourceGroupName, registryName, taskName, taskCreateParameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates a task for a container registry with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskCreateParameters The parameters for creating a task.
   *
   * @param {object} [taskCreateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskCreateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskCreateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskCreateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} taskCreateParameters.platform The platform properties
   * against which the run has to happen.
   *
   * @param {string} taskCreateParameters.platform.os The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskCreateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskCreateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} taskCreateParameters.step The properties of a task step.
   *
   * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskCreateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskCreateParameters.trigger] The properties that describe
   * all triggers for the task.
   *
   * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
   * of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskCreateParameters.credentials] The properties that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskCreateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {string} taskCreateParameters.location The location of the resource.
   * This cannot be changed after the resource is created.
   *
   * @param {object} [taskCreateParameters.tags] The tags of the resource.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Task} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Task} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  create(resourceGroupName, registryName, taskName, taskCreateParameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._create(resourceGroupName, registryName, taskName, taskCreateParameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._create(resourceGroupName, registryName, taskName, taskCreateParameters, options, optionalCallback);
    }
  }

  /**
   * Deletes a specified task.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deleteMethodWithHttpOperationResponse(resourceGroupName, registryName, taskName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deleteMethod(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes a specified task.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deleteMethod(resourceGroupName, registryName, taskName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deleteMethod(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deleteMethod(resourceGroupName, registryName, taskName, options, optionalCallback);
    }
  }

  /**
   * Updates a task with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskUpdateParameters The parameters for updating a task.
   *
   * @param {object} [taskUpdateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskUpdateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskUpdateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} [taskUpdateParameters.platform] The platform properties
   * against which the run has to happen.
   *
   * @param {string} [taskUpdateParameters.platform.os] The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskUpdateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskUpdateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} [taskUpdateParameters.step] The properties for updating a
   * task step.
   *
   * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskUpdateParameters.trigger] The properties for updating
   * trigger properties.
   *
   * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
   * type of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskUpdateParameters.credentials] The parameters that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskUpdateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  updateWithHttpOperationResponse(resourceGroupName, registryName, taskName, taskUpdateParameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._update(resourceGroupName, registryName, taskName, taskUpdateParameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Updates a task with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskUpdateParameters The parameters for updating a task.
   *
   * @param {object} [taskUpdateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskUpdateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskUpdateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} [taskUpdateParameters.platform] The platform properties
   * against which the run has to happen.
   *
   * @param {string} [taskUpdateParameters.platform.os] The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskUpdateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskUpdateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} [taskUpdateParameters.step] The properties for updating a
   * task step.
   *
   * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskUpdateParameters.trigger] The properties for updating
   * trigger properties.
   *
   * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
   * type of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskUpdateParameters.credentials] The parameters that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskUpdateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Task} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Task} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  update(resourceGroupName, registryName, taskName, taskUpdateParameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._update(resourceGroupName, registryName, taskName, taskUpdateParameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._update(resourceGroupName, registryName, taskName, taskUpdateParameters, options, optionalCallback);
    }
  }

  /**
   * Returns a task with extended information that includes all secrets.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getDetailsWithHttpOperationResponse(resourceGroupName, registryName, taskName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getDetails(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Returns a task with extended information that includes all secrets.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Task} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Task} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getDetails(resourceGroupName, registryName, taskName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getDetails(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getDetails(resourceGroupName, registryName, taskName, options, optionalCallback);
    }
  }

  /**
   * Creates a task for a container registry with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskCreateParameters The parameters for creating a task.
   *
   * @param {object} [taskCreateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskCreateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskCreateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskCreateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} taskCreateParameters.platform The platform properties
   * against which the run has to happen.
   *
   * @param {string} taskCreateParameters.platform.os The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskCreateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskCreateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} taskCreateParameters.step The properties of a task step.
   *
   * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskCreateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskCreateParameters.trigger] The properties that describe
   * all triggers for the task.
   *
   * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
   * of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskCreateParameters.credentials] The properties that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskCreateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {string} taskCreateParameters.location The location of the resource.
   * This cannot be changed after the resource is created.
   *
   * @param {object} [taskCreateParameters.tags] The tags of the resource.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginCreateWithHttpOperationResponse(resourceGroupName, registryName, taskName, taskCreateParameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginCreate(resourceGroupName, registryName, taskName, taskCreateParameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates a task for a container registry with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskCreateParameters The parameters for creating a task.
   *
   * @param {object} [taskCreateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskCreateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskCreateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskCreateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskCreateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskCreateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} taskCreateParameters.platform The platform properties
   * against which the run has to happen.
   *
   * @param {string} taskCreateParameters.platform.os The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskCreateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskCreateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskCreateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskCreateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskCreateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} taskCreateParameters.step The properties of a task step.
   *
   * @param {string} [taskCreateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskCreateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskCreateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskCreateParameters.trigger] The properties that describe
   * all triggers for the task.
   *
   * @param {array} [taskCreateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskCreateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskCreateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * taskCreateParameters.trigger.baseImageTrigger.baseImageTriggerType The type
   * of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskCreateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskCreateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskCreateParameters.credentials] The properties that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskCreateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskCreateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskCreateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {string} taskCreateParameters.location The location of the resource.
   * This cannot be changed after the resource is created.
   *
   * @param {object} [taskCreateParameters.tags] The tags of the resource.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Task} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Task} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginCreate(resourceGroupName, registryName, taskName, taskCreateParameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginCreate(resourceGroupName, registryName, taskName, taskCreateParameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginCreate(resourceGroupName, registryName, taskName, taskCreateParameters, options, optionalCallback);
    }
  }

  /**
   * Deletes a specified task.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginDeleteMethodWithHttpOperationResponse(resourceGroupName, registryName, taskName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginDeleteMethod(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes a specified task.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginDeleteMethod(resourceGroupName, registryName, taskName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginDeleteMethod(resourceGroupName, registryName, taskName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginDeleteMethod(resourceGroupName, registryName, taskName, options, optionalCallback);
    }
  }

  /**
   * Updates a task with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskUpdateParameters The parameters for updating a task.
   *
   * @param {object} [taskUpdateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskUpdateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskUpdateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} [taskUpdateParameters.platform] The platform properties
   * against which the run has to happen.
   *
   * @param {string} [taskUpdateParameters.platform.os] The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskUpdateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskUpdateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} [taskUpdateParameters.step] The properties for updating a
   * task step.
   *
   * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskUpdateParameters.trigger] The properties for updating
   * trigger properties.
   *
   * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
   * type of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskUpdateParameters.credentials] The parameters that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskUpdateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Task>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginUpdateWithHttpOperationResponse(resourceGroupName, registryName, taskName, taskUpdateParameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginUpdate(resourceGroupName, registryName, taskName, taskUpdateParameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Updates a task with the specified parameters.
   *
   * @param {string} resourceGroupName The name of the resource group to which
   * the container registry belongs.
   *
   * @param {string} registryName The name of the container registry.
   *
   * @param {string} taskName The name of the container registry task.
   *
   * @param {object} taskUpdateParameters The parameters for updating a task.
   *
   * @param {object} [taskUpdateParameters.identity] Identity for the resource.
   *
   * @param {string} [taskUpdateParameters.identity.principalId] The principal ID
   * of resource identity.
   *
   * @param {string} [taskUpdateParameters.identity.tenantId] The tenant ID of
   * resource.
   *
   * @param {string} [taskUpdateParameters.identity.type] The identity type.
   * Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
   *
   * @param {object} [taskUpdateParameters.identity.userAssignedIdentities] The
   * list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {string} [taskUpdateParameters.status] The current status of task.
   * Possible values include: 'Disabled', 'Enabled'
   *
   * @param {object} [taskUpdateParameters.platform] The platform properties
   * against which the run has to happen.
   *
   * @param {string} [taskUpdateParameters.platform.os] The operating system type
   * required for the run. Possible values include: 'Windows', 'Linux'
   *
   * @param {string} [taskUpdateParameters.platform.architecture] The OS
   * architecture. Possible values include: 'amd64', 'x86', 'arm'
   *
   * @param {string} [taskUpdateParameters.platform.variant] Variant of the CPU.
   * Possible values include: 'v6', 'v7', 'v8'
   *
   * @param {object} [taskUpdateParameters.agentConfiguration] The machine
   * configuration of the run agent.
   *
   * @param {number} [taskUpdateParameters.agentConfiguration.cpu] The CPU
   * configuration in terms of number of cores required for the run.
   *
   * @param {number} [taskUpdateParameters.timeout] Run timeout in seconds.
   *
   * @param {object} [taskUpdateParameters.step] The properties for updating a
   * task step.
   *
   * @param {string} [taskUpdateParameters.step.contextPath] The URL(absolute or
   * relative) of the source context for the task step.
   *
   * @param {string} [taskUpdateParameters.step.contextAccessToken] The token
   * (git PAT or SAS token of storage account blob) associated with the context
   * for a step.
   *
   * @param {string} taskUpdateParameters.step.type Polymorphic Discriminator
   *
   * @param {object} [taskUpdateParameters.trigger] The properties for updating
   * trigger properties.
   *
   * @param {array} [taskUpdateParameters.trigger.timerTriggers] The collection
   * of timer triggers.
   *
   * @param {array} [taskUpdateParameters.trigger.sourceTriggers] The collection
   * of triggers based on source code repository.
   *
   * @param {object} [taskUpdateParameters.trigger.baseImageTrigger] The trigger
   * based on base image dependencies.
   *
   * @param {string}
   * [taskUpdateParameters.trigger.baseImageTrigger.baseImageTriggerType] The
   * type of the auto trigger for base image dependency updates. Possible values
   * include: 'All', 'Runtime'
   *
   * @param {string} [taskUpdateParameters.trigger.baseImageTrigger.status] The
   * current status of trigger. Possible values include: 'Disabled', 'Enabled'
   *
   * @param {string} taskUpdateParameters.trigger.baseImageTrigger.name The name
   * of the trigger.
   *
   * @param {object} [taskUpdateParameters.credentials] The parameters that
   * describes a set of credentials that will be used when this run is invoked.
   *
   * @param {object} [taskUpdateParameters.credentials.sourceRegistry] Describes
   * the credential parameters for accessing the source registry.
   *
   * @param {string} [taskUpdateParameters.credentials.sourceRegistry.loginMode]
   * The authentication mode which determines the source registry login scope.
   * The credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   *
   * @param {object} [taskUpdateParameters.credentials.customRegistries]
   * Describes the credential parameters for accessing other custom registries.
   * The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   *
   * @param {object} [taskUpdateParameters.tags] The ARM resource tags.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Task} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Task} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginUpdate(resourceGroupName, registryName, taskName, taskUpdateParameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginUpdate(resourceGroupName, registryName, taskName, taskUpdateParameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginUpdate(resourceGroupName, registryName, taskName, taskUpdateParameters, options, optionalCallback);
    }
  }

  /**
   * Lists all the tasks for a specified container registry.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<TaskListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listNextWithHttpOperationResponse(nextPageLink, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listNext(nextPageLink, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Lists all the tasks for a specified container registry.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {TaskListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link TaskListResult} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listNext(nextPageLink, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listNext(nextPageLink, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listNext(nextPageLink, options, optionalCallback);
    }
  }

}

module.exports = Tasks;
