/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Principals class.
 * @constructor
 * User principals.
 *
 * @member {string} [upn] UPN of the user.
 * @member {string} [objectId] Object Id for the user
 */
export interface Principals {
  upn?: string;
  objectId?: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * The Resource model definition.
 *
 * @member {string} [id] Resource Id
 * @member {string} [name] Resource name
 * @member {string} [type] Resource type
 * @member {string} [location] Resource location
 * @member {object} [tags] Resource tags
 * @member {string} [etag] Resource etag
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the ADCCatalog class.
 * @constructor
 * Azure Data Catalog.
 *
 * @member {string} [sku] Azure data catalog SKU. Possible values include:
 * 'Free', 'Standard'
 * @member {number} [units] Azure data catalog units.
 * @member {array} [admins] Azure data catalog admin list.
 * @member {array} [users] Azure data catalog user list.
 * @member {boolean} [successfullyProvisioned] Azure data catalog provision
 * status.
 * @member {boolean} [enableAutomaticUnitAdjustment] Automatic unit adjustment
 * enabled or not.
 */
export interface ADCCatalog extends Resource {
  sku?: string;
  units?: number;
  admins?: Principals[];
  users?: Principals[];
  successfullyProvisioned?: boolean;
  enableAutomaticUnitAdjustment?: boolean;
}

/**
 * @class
 * Initializes a new instance of the ADCCatalogsListResult class.
 * @constructor
 * The response from the List Azure Data Catalog operation.
 *
 * @member {array} [value] the list of Azure Data Catalogs.
 */
export interface ADCCatalogsListResult {
  value?: ADCCatalog[];
}

/**
 * @class
 * Initializes a new instance of the OperationDisplayInfo class.
 * @constructor
 * The operation supported by Azure Data Catalog Service.
 *
 * @member {string} [description] The description of the operation.
 * @member {string} [operation] The action that users can perform, based on
 * their permission level.
 * @member {string} [provider] Service provider: Azure Data Catalog Service.
 * @member {string} [resource] Resource on which the operation is performed.
 */
export interface OperationDisplayInfo {
  description?: string;
  operation?: string;
  provider?: string;
  resource?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationEntity class.
 * @constructor
 * The operation supported by Azure Data Catalog Service.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}.
 * @member {object} [display] The operation supported by Azure Data Catalog
 * Service.
 * @member {string} [display.description] The description of the operation.
 * @member {string} [display.operation] The action that users can perform,
 * based on their permission level.
 * @member {string} [display.provider] Service provider: Azure Data Catalog
 * Service.
 * @member {string} [display.resource] Resource on which the operation is
 * performed.
 */
export interface OperationEntity {
  name?: string;
  display?: OperationDisplayInfo;
}

/**
 * @class
 * Initializes a new instance of the OperationEntityListResult class.
 * @constructor
 * The list of Azure data catalog service operation response.
 *
 * @member {array} [value] The list of operations.
 */
export interface OperationEntityListResult {
  value?: OperationEntity[];
}

