/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * ADCOperations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DataCatalogRestClient.
 */
export interface ADCOperations {


    /**
     * Lists all the available Azure Data Catalog service operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationEntityListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationEntityListResult>>;

    /**
     * Lists all the available Azure Data Catalog service operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationEntityListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationEntityListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationEntityListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationEntityListResult>;
    list(callback: ServiceCallback<models.OperationEntityListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationEntityListResult>): void;
}

/**
 * @class
 * ADCCatalogs
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DataCatalogRestClient.
 */
export interface ADCCatalogs {


    /**
     * @summary List catalogs in Resource Group (GET Resources)
     *
     * The List catalogs in Resource Group operation lists all the Azure Data
     * Catalogs available under the given resource group.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ADCCatalogsListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listtByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ADCCatalogsListResult>>;

    /**
     * @summary List catalogs in Resource Group (GET Resources)
     *
     * The List catalogs in Resource Group operation lists all the Azure Data
     * Catalogs available under the given resource group.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ADCCatalogsListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ADCCatalogsListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ADCCatalogsListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listtByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ADCCatalogsListResult>;
    listtByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.ADCCatalogsListResult>): void;
    listtByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ADCCatalogsListResult>): void;


    /**
     * @summary Create or Update Azure Data Catalog service (PUT Resource)
     *
     * The Create Azure Data Catalog service operation creates a new data catalog
     * service with the specified parameters. If the specific service already
     * exists, then any patchable properties will be updated and any immutable
     * properties will remain unchanged.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} properties Properties supplied to the Create or Update a
     * data catalog.
     *
     * @param {string} [properties.sku] Azure data catalog SKU. Possible values
     * include: 'Free', 'Standard'
     *
     * @param {number} [properties.units] Azure data catalog units.
     *
     * @param {array} [properties.admins] Azure data catalog admin list.
     *
     * @param {array} [properties.users] Azure data catalog user list.
     *
     * @param {boolean} [properties.successfullyProvisioned] Azure data catalog
     * provision status.
     *
     * @param {boolean} [properties.enableAutomaticUnitAdjustment] Automatic unit
     * adjustment enabled or not.
     *
     * @param {string} [properties.location] Resource location
     *
     * @param {object} [properties.tags] Resource tags
     *
     * @param {string} [properties.etag] Resource etag
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ADCCatalog>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, properties: models.ADCCatalog, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ADCCatalog>>;

    /**
     * @summary Create or Update Azure Data Catalog service (PUT Resource)
     *
     * The Create Azure Data Catalog service operation creates a new data catalog
     * service with the specified parameters. If the specific service already
     * exists, then any patchable properties will be updated and any immutable
     * properties will remain unchanged.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} properties Properties supplied to the Create or Update a
     * data catalog.
     *
     * @param {string} [properties.sku] Azure data catalog SKU. Possible values
     * include: 'Free', 'Standard'
     *
     * @param {number} [properties.units] Azure data catalog units.
     *
     * @param {array} [properties.admins] Azure data catalog admin list.
     *
     * @param {array} [properties.users] Azure data catalog user list.
     *
     * @param {boolean} [properties.successfullyProvisioned] Azure data catalog
     * provision status.
     *
     * @param {boolean} [properties.enableAutomaticUnitAdjustment] Automatic unit
     * adjustment enabled or not.
     *
     * @param {string} [properties.location] Resource location
     *
     * @param {object} [properties.tags] Resource tags
     *
     * @param {string} [properties.etag] Resource etag
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ADCCatalog} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ADCCatalog} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ADCCatalog} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, properties: models.ADCCatalog, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ADCCatalog>;
    createOrUpdate(resourceGroupName: string, properties: models.ADCCatalog, callback: ServiceCallback<models.ADCCatalog>): void;
    createOrUpdate(resourceGroupName: string, properties: models.ADCCatalog, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ADCCatalog>): void;


    /**
     * @summary Get Azure Data Catalog service (GET Resources)
     *
     * The Get Azure Data Catalog Service operation retrieves a json representation
     * of the data catalog.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ADCCatalog>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ADCCatalog>>;

    /**
     * @summary Get Azure Data Catalog service (GET Resources)
     *
     * The Get Azure Data Catalog Service operation retrieves a json representation
     * of the data catalog.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ADCCatalog} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ADCCatalog} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ADCCatalog} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ADCCatalog>;
    get(resourceGroupName: string, callback: ServiceCallback<models.ADCCatalog>): void;
    get(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ADCCatalog>): void;


    /**
     * @summary Delete Azure Data Catalog Service (DELETE Resource)
     *
     * The Delete Azure Data Catalog Service operation deletes an existing data
     * catalog.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Delete Azure Data Catalog Service (DELETE Resource)
     *
     * The Delete Azure Data Catalog Service operation deletes an existing data
     * catalog.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Update Azure Data Catalog Service (PATCH Resource)
     *
     * The Update Azure Data Catalog Service operation can be used to update the
     * existing deployment. The update call only supports the properties listed in
     * the PATCH body.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} properties Properties supplied to the Update a data catalog.
     *
     * @param {string} [properties.sku] Azure data catalog SKU. Possible values
     * include: 'Free', 'Standard'
     *
     * @param {number} [properties.units] Azure data catalog units.
     *
     * @param {array} [properties.admins] Azure data catalog admin list.
     *
     * @param {array} [properties.users] Azure data catalog user list.
     *
     * @param {boolean} [properties.successfullyProvisioned] Azure data catalog
     * provision status.
     *
     * @param {boolean} [properties.enableAutomaticUnitAdjustment] Automatic unit
     * adjustment enabled or not.
     *
     * @param {string} [properties.location] Resource location
     *
     * @param {object} [properties.tags] Resource tags
     *
     * @param {string} [properties.etag] Resource etag
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ADCCatalog>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, properties: models.ADCCatalog, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ADCCatalog>>;

    /**
     * @summary Update Azure Data Catalog Service (PATCH Resource)
     *
     * The Update Azure Data Catalog Service operation can be used to update the
     * existing deployment. The update call only supports the properties listed in
     * the PATCH body.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} properties Properties supplied to the Update a data catalog.
     *
     * @param {string} [properties.sku] Azure data catalog SKU. Possible values
     * include: 'Free', 'Standard'
     *
     * @param {number} [properties.units] Azure data catalog units.
     *
     * @param {array} [properties.admins] Azure data catalog admin list.
     *
     * @param {array} [properties.users] Azure data catalog user list.
     *
     * @param {boolean} [properties.successfullyProvisioned] Azure data catalog
     * provision status.
     *
     * @param {boolean} [properties.enableAutomaticUnitAdjustment] Automatic unit
     * adjustment enabled or not.
     *
     * @param {string} [properties.location] Resource location
     *
     * @param {object} [properties.tags] Resource tags
     *
     * @param {string} [properties.etag] Resource etag
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ADCCatalog} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ADCCatalog} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ADCCatalog} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, properties: models.ADCCatalog, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ADCCatalog>;
    update(resourceGroupName: string, properties: models.ADCCatalog, callback: ServiceCallback<models.ADCCatalog>): void;
    update(resourceGroupName: string, properties: models.ADCCatalog, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ADCCatalog>): void;


    /**
     * @summary Delete Azure Data Catalog Service (DELETE Resource)
     *
     * The Delete Azure Data Catalog Service operation deletes an existing data
     * catalog.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Delete Azure Data Catalog Service (DELETE Resource)
     *
     * The Delete Azure Data Catalog Service operation deletes an existing data
     * catalog.
     *
     * @param {string} resourceGroupName The name of the resource group within the
     * user's subscription. The name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}
