/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Identity class.
 * @constructor
 * Identity for the resource.
 *
 * @member {string} type The identity type.
 * @member {array} identityIds The list of identities.
 */
export interface Identity {
  type: string;
  identityIds: string[];
}

/**
 * @class
 * Initializes a new instance of the PrePostStep class.
 * @constructor
 * The properties that define a step.
 *
 * @member {string} stepId The resource Id of the step to be run.
 */
export interface PrePostStep {
  stepId: string;
}

/**
 * @class
 * Initializes a new instance of the Step class.
 * @constructor
 * The properties that define an Azure Deployment Manager step.
 *
 * @member {string} name The name of the step group.
 * @member {array} [dependsOnStepGroups] The list of step group names on which
 * this step group depends on.
 * @member {array} [preDeploymentSteps] The list of steps to be run before
 * deploying the target.
 * @member {string} deploymentTargetId The resource Id of service unit to be
 * deployed. The service unit should be from the service topology referenced in
 * targetServiceTopologyId
 * @member {array} [postDeploymentSteps] The list of steps to be run after
 * deploying the target.
 */
export interface Step {
  name: string;
  dependsOnStepGroups?: string[];
  preDeploymentSteps?: PrePostStep[];
  deploymentTargetId: string;
  postDeploymentSteps?: PrePostStep[];
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * @member {string} [id] Fully qualified resource Id for the resource. Ex -
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}
 * @member {string} [name] The name of the resource
 * @member {string} [type] The type of the resource. Ex-
 * Microsoft.Compute/virtualMachines or Microsoft.Storage/storageAccounts.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the TrackedResource class.
 * @constructor
 * The resource model definition for a ARM tracked top level resource
 *
 * @member {object} [tags] Resource tags.
 * @member {string} location The geo-location where the resource lives
 */
export interface TrackedResource extends Resource {
  tags?: { [propertyName: string]: string };
  location: string;
}

/**
 * @class
 * Initializes a new instance of the RolloutRequest class.
 * @constructor
 * Defines the PUT rollout request body.
 *
 * @member {object} identity Identity for the resource.
 * @member {string} [identity.type] The identity type.
 * @member {array} [identity.identityIds] The list of identities.
 * @member {string} buildVersion The version of the build being deployed.
 * @member {string} [artifactSourceId] The reference to the artifact source
 * resource Id where the payload is located.
 * @member {string} targetServiceTopologyId The resource Id of the service
 * topology from which service units are being referenced in step groups to be
 * deployed.
 * @member {array} stepGroups The list of step groups that define the
 * orchestration.
 */
export interface RolloutRequest extends TrackedResource {
  identity: Identity;
  buildVersion: string;
  artifactSourceId?: string;
  targetServiceTopologyId: string;
  stepGroups: Step[];
}

/**
 * @class
 * Initializes a new instance of the ArtifactSource class.
 * @constructor
 * The resource that defines the source location where the artifacts are
 * located.
 *
 * @member {string} sourceType The type of artifact source used.
 * @member {string} [artifactRoot] The path from the location that the
 * 'authentication' property [say, a SAS URI to the blob container] refers to,
 * to the location of the artifacts. This can be used to differentiate
 * different versions of the artifacts. Or, different types of artifacts like
 * binaries or templates. The location referenced by the authentication
 * property concatenated with this optional artifactRoot path forms the
 * artifact source location where the artifacts are expected to be found.
 * @member {object} authentication The authentication method to use to access
 * the artifact source.
 * @member {string} [authentication.type] Polymorphic Discriminator
 */
export interface ArtifactSource extends TrackedResource {
  sourceType: string;
  artifactRoot?: string;
  authentication: Authentication;
}

/**
 * @class
 * Initializes a new instance of the Authentication class.
 * @constructor
 * Defines the authentication method and properties to access the artifacts.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface Authentication {
  type: string;
}

/**
 * @class
 * Initializes a new instance of the ArtifactSourcePropertiesModel class.
 * @constructor
 * The properties that define the source location where the artifacts are
 * located.
 *
 * @member {string} sourceType The type of artifact source used.
 * @member {string} [artifactRoot] The path from the location that the
 * 'authentication' property [say, a SAS URI to the blob container] refers to,
 * to the location of the artifacts. This can be used to differentiate
 * different versions of the artifacts. Or, different types of artifacts like
 * binaries or templates. The location referenced by the authentication
 * property concatenated with this optional artifactRoot path forms the
 * artifact source location where the artifacts are expected to be found.
 * @member {object} authentication The authentication method to use to access
 * the artifact source.
 * @member {string} [authentication.type] Polymorphic Discriminator
 */
export interface ArtifactSourcePropertiesModel {
  sourceType: string;
  artifactRoot?: string;
  authentication: Authentication;
}

/**
 * @class
 * Initializes a new instance of the SasAuthentication class.
 * @constructor
 * Defines the properties to access the artifacts using an Azure Storage SAS
 * URI.
 *
 * @member {string} sasUri The SAS URI to the Azure Storage blob container. Any
 * offset from the root of the container to where the artifacts are located can
 * be defined in the artifactRoot.
 */
export interface SasAuthentication extends Authentication {
  sasUri: string;
}

/**
 * @class
 * Initializes a new instance of the CloudErrorBody class.
 * @constructor
 * Detailed error information of any failure.
 *
 * @member {string} [code] Error code string.
 * @member {string} [message] Descriptive error information.
 * @member {string} [target] Error target
 * @member {array} [details] More detailed error information.
 */
export interface CloudErrorBody {
  readonly code?: string;
  readonly message?: string;
  target?: string;
  details?: CloudErrorBody[];
}

/**
 * @class
 * Initializes a new instance of the RolloutOperationInfo class.
 * @constructor
 * Detailed runtime information of the rollout.
 *
 * @member {number} [retryAttempt] The ordinal count of the number of retry
 * attempts on a rollout. 0 if no retries of the rollout have been performed.
 * If the rollout is updated with a PUT, this count is reset to 0.
 * @member {boolean} [skipSucceededOnRetry] True, if all steps that succeeded
 * on the previous run/attempt were chosen to be skipped in this retry attempt.
 * False, otherwise.
 * @member {date} [startTime] The start time of the rollout in UTC.
 * @member {date} [endTime] The start time of the rollout in UTC. This property
 * will not be set if the rollout has not completed yet.
 * @member {object} [error] The detailed error information for any failure.
 * @member {string} [error.code] Error code string.
 * @member {string} [error.message] Descriptive error information.
 * @member {string} [error.target] Error target
 * @member {array} [error.details] More detailed error information.
 */
export interface RolloutOperationInfo {
  readonly retryAttempt?: number;
  readonly skipSucceededOnRetry?: boolean;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly error?: CloudErrorBody;
}

/**
 * @class
 * Initializes a new instance of the StepOperationInfo class.
 * @constructor
 * Detailed information of a specific step run.
 *
 * @member {string} [deploymentName] The name of the ARM deployment initiated
 * as part of the step.
 * @member {string} [correlationId] Unique identifier to track the request for
 * ARM-based resources.
 * @member {date} [startTime] Start time of the action in UTC.
 * @member {date} [endTime] End time of the action in UTC.
 * @member {date} [lastUpdatedTime] Last time in UTC this operation was
 * updated.
 * @member {object} [error] The errors, if any, for the action.
 * @member {string} [error.code] Error code string.
 * @member {string} [error.message] Descriptive error information.
 * @member {string} [error.target] Error target
 * @member {array} [error.details] More detailed error information.
 */
export interface StepOperationInfo {
  readonly deploymentName?: string;
  readonly correlationId?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly lastUpdatedTime?: Date;
  error?: CloudErrorBody;
}

/**
 * @class
 * Initializes a new instance of the ResourceOperation class.
 * @constructor
 * Individial resource operation information.
 *
 * @member {string} [resourceName] Name of the resource as specified in the
 * artifacts. For ARM resources, this is the name of the resource specified in
 * the template.
 * @member {string} [operationId] Unique identifier of the operation. For ARM
 * resources, this is the operationId obtained from ARM service.
 * @member {string} [resourceType] Type of the resource as specified in the
 * artifacts. For ARM resources, this is the type of the resource specified in
 * the template.
 * @member {string} [provisioningState] State of the resource deployment. For
 * ARM resources, this is the current provisioning state of the resource.
 * @member {string} [statusMessage] Descriptive information of the resource
 * operation.
 * @member {string} [statusCode] Http status code of the operation.
 */
export interface ResourceOperation {
  resourceName?: string;
  readonly operationId?: string;
  resourceType?: string;
  readonly provisioningState?: string;
  readonly statusMessage?: string;
  readonly statusCode?: string;
}

/**
 * @class
 * Initializes a new instance of the Message class.
 * @constructor
 * Supplementary contextual messages during a rollout.
 *
 * @member {date} [timeStamp] Time in UTC this message was provided.
 * @member {string} [message] The actual message text.
 */
export interface Message {
  readonly timeStamp?: Date;
  readonly message?: string;
}

/**
 * @class
 * Initializes a new instance of the RolloutStep class.
 * @constructor
 * Defines a specific step on a target service unit.
 *
 * @member {string} name Name of the step.
 * @member {string} [status] Current state of the step.
 * @member {string} [stepGroup] The step group the current step is part of.
 * @member {object} [operationInfo] Detailed information of specific action
 * execution.
 * @member {string} [operationInfo.deploymentName] The name of the ARM
 * deployment initiated as part of the step.
 * @member {string} [operationInfo.correlationId] Unique identifier to track
 * the request for ARM-based resources.
 * @member {date} [operationInfo.startTime] Start time of the action in UTC.
 * @member {date} [operationInfo.endTime] End time of the action in UTC.
 * @member {date} [operationInfo.lastUpdatedTime] Last time in UTC this
 * operation was updated.
 * @member {object} [operationInfo.error] The errors, if any, for the action.
 * @member {string} [operationInfo.error.code] Error code string.
 * @member {string} [operationInfo.error.message] Descriptive error
 * information.
 * @member {string} [operationInfo.error.target] Error target
 * @member {array} [operationInfo.error.details] More detailed error
 * information.
 * @member {array} [resourceOperations] Set of resource operations that were
 * performed, if any, on an Azure resource.
 * @member {array} [messages] Supplementary informative messages during
 * rollout.
 */
export interface RolloutStep {
  name: string;
  readonly status?: string;
  stepGroup?: string;
  readonly operationInfo?: StepOperationInfo;
  readonly resourceOperations?: ResourceOperation[];
  readonly messages?: Message[];
}

/**
 * @class
 * Initializes a new instance of the ServiceUnitProperties class.
 * @constructor
 * Defines the properties of a service unit.
 *
 * @member {string} targetResourceGroup The Azure Resource Group to which the
 * resources in the service unit belong to or should be deployed to.
 * @member {string} deploymentMode Describes the type of ARM deployment to be
 * performed on the resource. Possible values include: 'Incremental',
 * 'Complete'
 * @member {object} [artifacts] The artifacts for the service unit.
 * @member {string} [artifacts.templateUri] The full URI of the ARM template
 * file with the SAS token.
 * @member {string} [artifacts.parametersUri] The full URI of the ARM
 * parameters file with the SAS token.
 * @member {string} [artifacts.templateArtifactSourceRelativePath] The path to
 * the ARM template file relative to the artifact source.
 * @member {string} [artifacts.parametersArtifactSourceRelativePath] The path
 * to the ARM parameters file relative to the artifact source.
 */
export interface ServiceUnitProperties {
  targetResourceGroup: string;
  deploymentMode: string;
  artifacts?: ServiceUnitArtifacts;
}

/**
 * @class
 * Initializes a new instance of the ServiceUnit class.
 * @constructor
 * Defines a service unit.
 *
 * @member {string} [name] Name of the service unit.
 * @member {array} [steps] Detailed step information, if present.
 */
export interface ServiceUnit extends ServiceUnitProperties {
  name?: string;
  steps?: RolloutStep[];
}

/**
 * @class
 * Initializes a new instance of the ServiceProperties class.
 * @constructor
 * The properties of a service.
 *
 * @member {string} targetLocation The Azure location to which the resources in
 * the service belong to or should be deployed to.
 * @member {string} targetSubscriptionId The subscription to which the
 * resources in the service belong to or should be deployed to.
 */
export interface ServiceProperties {
  targetLocation: string;
  targetSubscriptionId: string;
}

/**
 * @class
 * Initializes a new instance of the Service class.
 * @constructor
 * Defines a service.
 *
 * @member {string} [name] Name of the service.
 * @member {array} [serviceUnits] The detailed information about the units that
 * make up the service.
 */
export interface Service extends ServiceProperties {
  name?: string;
  serviceUnits?: ServiceUnit[];
}

/**
 * @class
 * Initializes a new instance of the Rollout class.
 * @constructor
 * Defines the rollout.
 *
 * @member {object} [identity] Identity for the resource.
 * @member {string} [identity.type] The identity type.
 * @member {array} [identity.identityIds] The list of identities.
 * @member {string} buildVersion The version of the build being deployed.
 * @member {string} [artifactSourceId] The reference to the artifact source
 * resource Id where the payload is located.
 * @member {string} targetServiceTopologyId The resource Id of the service
 * topology from which service units are being referenced in step groups to be
 * deployed.
 * @member {array} stepGroups The list of step groups that define the
 * orchestration.
 * @member {string} [status] The current status of the rollout.
 * @member {number} [totalRetryAttempts] The cardinal count of total number of
 * retries performed on the rollout at a given time.
 * @member {object} [operationInfo] Operational information of the rollout.
 * @member {number} [operationInfo.retryAttempt] The ordinal count of the
 * number of retry attempts on a rollout. 0 if no retries of the rollout have
 * been performed. If the rollout is updated with a PUT, this count is reset to
 * 0.
 * @member {boolean} [operationInfo.skipSucceededOnRetry] True, if all steps
 * that succeeded on the previous run/attempt were chosen to be skipped in this
 * retry attempt. False, otherwise.
 * @member {date} [operationInfo.startTime] The start time of the rollout in
 * UTC.
 * @member {date} [operationInfo.endTime] The start time of the rollout in UTC.
 * This property will not be set if the rollout has not completed yet.
 * @member {object} [operationInfo.error] The detailed error information for
 * any failure.
 * @member {string} [operationInfo.error.code] Error code string.
 * @member {string} [operationInfo.error.message] Descriptive error
 * information.
 * @member {string} [operationInfo.error.target] Error target
 * @member {array} [operationInfo.error.details] More detailed error
 * information.
 * @member {array} [services] The detailed information on the services being
 * deployed.
 */
export interface Rollout extends TrackedResource {
  identity?: Identity;
  buildVersion: string;
  artifactSourceId?: string;
  targetServiceTopologyId: string;
  stepGroups: Step[];
  readonly status?: string;
  readonly totalRetryAttempts?: number;
  readonly operationInfo?: RolloutOperationInfo;
  readonly services?: Service[];
}

/**
 * @class
 * Initializes a new instance of the RolloutPropertiesModel class.
 * @constructor
 * Defines the properties of a rollout.
 *
 * @member {string} [status] The current status of the rollout.
 * @member {number} [totalRetryAttempts] The cardinal count of total number of
 * retries performed on the rollout at a given time.
 * @member {object} [operationInfo] Operational information of the rollout.
 * @member {number} [operationInfo.retryAttempt] The ordinal count of the
 * number of retry attempts on a rollout. 0 if no retries of the rollout have
 * been performed. If the rollout is updated with a PUT, this count is reset to
 * 0.
 * @member {boolean} [operationInfo.skipSucceededOnRetry] True, if all steps
 * that succeeded on the previous run/attempt were chosen to be skipped in this
 * retry attempt. False, otherwise.
 * @member {date} [operationInfo.startTime] The start time of the rollout in
 * UTC.
 * @member {date} [operationInfo.endTime] The start time of the rollout in UTC.
 * This property will not be set if the rollout has not completed yet.
 * @member {object} [operationInfo.error] The detailed error information for
 * any failure.
 * @member {string} [operationInfo.error.code] Error code string.
 * @member {string} [operationInfo.error.message] Descriptive error
 * information.
 * @member {string} [operationInfo.error.target] Error target
 * @member {array} [operationInfo.error.details] More detailed error
 * information.
 * @member {array} [services] The detailed information on the services being
 * deployed.
 */
export interface RolloutPropertiesModel {
  readonly status?: string;
  readonly totalRetryAttempts?: number;
  readonly operationInfo?: RolloutOperationInfo;
  readonly services?: Service[];
}

/**
 * @class
 * Initializes a new instance of the ServiceTopologyResource class.
 * @constructor
 * The resource representation of a service topology.
 *
 * @member {string} [artifactSourceId] The resource Id of the artifact source
 * that contains the artifacts that can be referenced in the service units.
 */
export interface ServiceTopologyResource extends TrackedResource {
  artifactSourceId?: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceTopologyProperties class.
 * @constructor
 * The properties of a service topology.
 *
 * @member {string} [artifactSourceId] The resource Id of the artifact source
 * that contains the artifacts that can be referenced in the service units.
 */
export interface ServiceTopologyProperties {
  artifactSourceId?: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceResource class.
 * @constructor
 * The resource representation of a service in a service topology.
 *
 * @member {string} targetLocation The Azure location to which the resources in
 * the service belong to or should be deployed to.
 * @member {string} targetSubscriptionId The subscription to which the
 * resources in the service belong to or should be deployed to.
 */
export interface ServiceResource extends TrackedResource {
  targetLocation: string;
  targetSubscriptionId: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceUnitResource class.
 * @constructor
 * Represents the response of a service unit resource.
 *
 * @member {string} targetResourceGroup The Azure Resource Group to which the
 * resources in the service unit belong to or should be deployed to.
 * @member {string} deploymentMode Describes the type of ARM deployment to be
 * performed on the resource. Possible values include: 'Incremental',
 * 'Complete'
 * @member {object} [artifacts] The artifacts for the service unit.
 * @member {string} [artifacts.templateUri] The full URI of the ARM template
 * file with the SAS token.
 * @member {string} [artifacts.parametersUri] The full URI of the ARM
 * parameters file with the SAS token.
 * @member {string} [artifacts.templateArtifactSourceRelativePath] The path to
 * the ARM template file relative to the artifact source.
 * @member {string} [artifacts.parametersArtifactSourceRelativePath] The path
 * to the ARM parameters file relative to the artifact source.
 */
export interface ServiceUnitResource extends TrackedResource {
  targetResourceGroup: string;
  deploymentMode: string;
  artifacts?: ServiceUnitArtifacts;
}

/**
 * @class
 * Initializes a new instance of the ServiceUnitArtifacts class.
 * @constructor
 * Defines the artifacts of a service unit.
 *
 * @member {string} [templateUri] The full URI of the ARM template file with
 * the SAS token.
 * @member {string} [parametersUri] The full URI of the ARM parameters file
 * with the SAS token.
 * @member {string} [templateArtifactSourceRelativePath] The path to the ARM
 * template file relative to the artifact source.
 * @member {string} [parametersArtifactSourceRelativePath] The path to the ARM
 * parameters file relative to the artifact source.
 */
export interface ServiceUnitArtifacts {
  templateUri?: string;
  parametersUri?: string;
  templateArtifactSourceRelativePath?: string;
  parametersArtifactSourceRelativePath?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationDetail class.
 * @constructor
 * The detail about an operation.
 *
 * @member {string} [provider] The name of the provider that supports the
 * operation.
 * @member {string} [resource] The resource type on which this operation can be
 * performed.
 * @member {string} [operation] The name of the operation.
 * @member {string} [description] The description of the operation.
 */
export interface OperationDetail {
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Represents an operation that can be performed on the service.
 *
 * @member {string} [name] The name of the operation.
 * @member {object} [display] The display name of the operation.
 * @member {string} [display.provider] The name of the provider that supports
 * the operation.
 * @member {string} [display.resource] The resource type on which this
 * operation can be performed.
 * @member {string} [display.operation] The name of the operation.
 * @member {string} [display.description] The description of the operation.
 * @member {string} [origin] The origin of the operation.
 * @member {object} [properties] The properties of the operation.
 */
export interface Operation {
  name?: string;
  display?: OperationDetail;
  origin?: string;
  properties?: any;
}

/**
 * @class
 * Initializes a new instance of the StepProperties class.
 * @constructor
 * The properties of a step resource.
 *
 * @member {string} stepType Polymorphic Discriminator
 */
export interface StepProperties {
  stepType: string;
}

/**
 * @class
 * Initializes a new instance of the StepResource class.
 * @constructor
 * The resource representation of a rollout step.
 *
 * @member {object} properties The properties that define the step.
 * @member {string} [properties.stepType] Polymorphic Discriminator
 */
export interface StepResource extends TrackedResource {
  properties: StepProperties;
}

/**
 * @class
 * Initializes a new instance of the WaitStepAttributes class.
 * @constructor
 * The parameters for the wait step.
 *
 * @member {string} duration The duration in ISO 8601 format of how long the
 * wait should be.
 */
export interface WaitStepAttributes {
  duration: string;
}

/**
 * @class
 * Initializes a new instance of the WaitStepProperties class.
 * @constructor
 * Defines the properties of a Wait step.
 *
 * @member {object} [attributes] The Wait attributes
 * @member {string} [attributes.duration] The duration in ISO 8601 format of
 * how long the wait should be.
 */
export interface WaitStepProperties extends StepProperties {
  attributes?: WaitStepAttributes;
}

/**
 * @class
 * Initializes a new instance of the ProxyResource class.
 * @constructor
 * The resource model definition for a ARM proxy resource. It will have
 * everything other than required location and tags
 *
 */
export interface ProxyResource extends Resource {
}

/**
 * @class
 * Initializes a new instance of the AzureEntityResource class.
 * @constructor
 * The resource model definition for a Azure Resource Manager resource with an
 * etag.
 *
 * @member {string} [etag] Resource Etag.
 */
export interface AzureEntityResource extends Resource {
  readonly etag?: string;
}

