/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The description of the IoTSpaces service.
 *
 * @extends models['Resource']
 */
class IoTSpacesDescription extends models['Resource'] {
  /**
   * Create a IoTSpacesDescription.
   * @member {object} [properties] The common properties of a IoTSpaces
   * service.
   * @member {string} [properties.provisioningState] The provisioning state.
   * Possible values include: 'Provisioning', 'Deleting', 'Succeeded',
   * 'Failed', 'Canceled'
   * @member {string} [properties.managementApiUrl] The management Api
   * endpoint.
   * @member {string} [properties.webPortalUrl] The management UI endpoint.
   * @member {object} [properties.storageContainer] The properties of the
   * designated storage container.
   * @member {string} [properties.storageContainer.connectionString] The
   * connection string of the storage account.
   * @member {string} [properties.storageContainer.subscriptionId] The
   * subscription identifier of the storage account.
   * @member {string} [properties.storageContainer.resourceGroup] The name of
   * the resource group of the storage account.
   * @member {string} [properties.storageContainer.containerName] The name of
   * storage container in the storage account.
   * @member {object} sku A valid instance SKU.
   * @member {string} [sku.name] The name of the SKU. Possible values include:
   * 'F1', 'S1', 'S2', 'S3'
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of IoTSpacesDescription
   *
   * @returns {object} metadata of IoTSpacesDescription
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'IoTSpacesDescription',
      type: {
        name: 'Composite',
        className: 'IoTSpacesDescription',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            constraints: {
              Pattern: '^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$'
            },
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          properties: {
            required: false,
            serializedName: 'properties',
            type: {
              name: 'Composite',
              className: 'IoTSpacesProperties'
            }
          },
          sku: {
            required: true,
            serializedName: 'sku',
            type: {
              name: 'Composite',
              className: 'IoTSpacesSkuInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = IoTSpacesDescription;
