/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents an image from the Azure Marketplace
 *
 * @extends models['Resource']
 */
class GalleryImage extends models['Resource'] {
  /**
   * Create a GalleryImage.
   * @property {string} [author] The author of the gallery image.
   * @property {date} [createdDate] The creation date of the gallery image.
   * @property {string} [description] The description of the gallery image.
   * @property {object} [imageReference] The image reference of the gallery
   * image.
   * @property {string} [imageReference.offer] The offer of the gallery image.
   * @property {string} [imageReference.publisher] The publisher of the gallery
   * image.
   * @property {string} [imageReference.sku] The SKU of the gallery image.
   * @property {string} [imageReference.osType] The OS type of the gallery
   * image.
   * @property {string} [imageReference.version] The version of the gallery
   * image.
   * @property {string} [icon] The icon of the gallery image.
   * @property {boolean} [isEnabled] Indicates whether this gallery image is
   * enabled.
   * @property {boolean} [isOverride] Indicates whether this gallery has been
   * overridden for this lab account
   * @property {string} [planId] The third party plan that applies to this
   * image
   * @property {boolean} [isPlanAuthorized] Indicates if the plan has been
   * authorized for programmatic deployment.
   * @property {string} [provisioningState] The provisioning status of the
   * resource.
   * @property {string} [uniqueIdentifier] The unique immutable identifier of a
   * resource (Guid).
   * @property {object} [latestOperationResult] The details of the latest
   * operation. ex: status, error
   * @property {string} [latestOperationResult.status] The current status of
   * the operation.
   * @property {string} [latestOperationResult.errorCode] Error code on
   * failure.
   * @property {string} [latestOperationResult.errorMessage] The error message.
   * @property {string} [latestOperationResult.requestUri] Request URI of the
   * operation.
   * @property {string} [latestOperationResult.httpMethod] The HttpMethod -
   * PUT/POST/DELETE for the operation.
   * @property {string} [latestOperationResult.operationUrl] The URL to use to
   * check long-running operation status
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of GalleryImage
   *
   * @returns {object} metadata of GalleryImage
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'GalleryImage',
      type: {
        name: 'Composite',
        className: 'GalleryImage',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          author: {
            required: false,
            readOnly: true,
            serializedName: 'properties.author',
            type: {
              name: 'String'
            }
          },
          createdDate: {
            required: false,
            readOnly: true,
            serializedName: 'properties.createdDate',
            type: {
              name: 'DateTime'
            }
          },
          description: {
            required: false,
            readOnly: true,
            serializedName: 'properties.description',
            type: {
              name: 'String'
            }
          },
          imageReference: {
            required: false,
            readOnly: true,
            serializedName: 'properties.imageReference',
            type: {
              name: 'Composite',
              className: 'GalleryImageReference'
            }
          },
          icon: {
            required: false,
            readOnly: true,
            serializedName: 'properties.icon',
            type: {
              name: 'String'
            }
          },
          isEnabled: {
            required: false,
            serializedName: 'properties.isEnabled',
            type: {
              name: 'Boolean'
            }
          },
          isOverride: {
            required: false,
            serializedName: 'properties.isOverride',
            type: {
              name: 'Boolean'
            }
          },
          planId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.planId',
            type: {
              name: 'String'
            }
          },
          isPlanAuthorized: {
            required: false,
            serializedName: 'properties.isPlanAuthorized',
            type: {
              name: 'Boolean'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          uniqueIdentifier: {
            required: false,
            serializedName: 'properties.uniqueIdentifier',
            type: {
              name: 'String'
            }
          },
          latestOperationResult: {
            required: false,
            readOnly: true,
            serializedName: 'properties.latestOperationResult',
            type: {
              name: 'Composite',
              className: 'LatestOperationResult'
            }
          }
        }
      }
    };
  }
}

module.exports = GalleryImage;
