/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Identity class.
 * @constructor
 * Describes an identity resource.
 *
 * @member {string} [id] The id of the created identity.
 * @member {string} [name] The name of the created identity.
 * @member {string} [location] The Azure region where the identity lives.
 * @member {object} [tags] Resource tags
 * @member {uuid} [tenantId] The id of the tenant which the identity belongs
 * to.
 * @member {uuid} [principalId] The id of the service principal object
 * associated with the created identity.
 * @member {uuid} [clientId] The id of the app associated with the identity.
 * This is a random generated UUID by MSI.
 * @member {string} [clientSecretUrl] The ManagedServiceIdentity DataPlane URL
 * that can be queried to obtain the identity credentials.
 * @member {string} [type] The type of resource i.e.
 * Microsoft.ManagedIdentity/userAssignedIdentities. Possible values include:
 * 'Microsoft.ManagedIdentity/userAssignedIdentities'
 */
export interface Identity extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
  readonly tenantId?: string;
  readonly principalId?: string;
  readonly clientId?: string;
  readonly clientSecretUrl?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * @summary Operation Display.
 *
 * The object that describes the operation.
 *
 * @member {string} [provider] Resource Provider Name. Friendly name of the
 * resource provider.
 * @member {string} [operation] Operation Type. The type of operation. For
 * example: read, write, delete.
 * @member {string} [resource] Resource Type. The resource type on which the
 * operation is performed.
 * @member {string} [description] Operation description. A description of the
 * operation.
 */
export interface OperationDisplay {
  provider?: string;
  operation?: string;
  resource?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * @summary Microsoft.ManagedIdentity Operation.
 *
 * Operation supported by the Microsoft.ManagedIdentity REST API.
 *
 * @member {string} [name] Operation Name. The name of the REST Operation. This
 * is of the format {provider}/{resource}/{operation}.
 * @member {object} [display] Operation Display. The object that describes the
 * operation.
 * @member {string} [display.provider] Friendly name of the resource provider.
 * @member {string} [display.operation] The type of operation. For example:
 * read, write, delete.
 * @member {string} [display.resource] The resource type on which the operation
 * is performed.
 * @member {string} [display.description] A description of the operation.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}


/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * @summary Operations List.
 *
 * A list of operations supported by Microsoft.ManagedIdentity Resource
 * Provider.
 *
 * @member {string} [nextLink] The url to get the next page of results, if any.
 */
export interface OperationListResult extends Array<Operation> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the UserAssignedIdentitiesListResult class.
 * @constructor
 * Values returned by the List operation.
 *
 * @member {string} [nextLink] The url to get the next page of results, if any.
 */
export interface UserAssignedIdentitiesListResult extends Array<Identity> {
  nextLink?: string;
}
