/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Capacities
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the PowerBIDedicatedManagementClient.
 */
export interface Capacities {


    /**
     * Gets details about the specified dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the dedicated capacity. It
     * must be a minimum of 3 characters, and a maximum of 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacity>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getDetailsWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacity>>;

    /**
     * Gets details about the specified dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the dedicated capacity. It
     * must be a minimum of 3 characters, and a maximum of 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacity} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacity} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacity} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getDetails(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacity>;
    getDetails(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<models.DedicatedCapacity>): void;
    getDetails(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacity>): void;


    /**
     * Provisions the specified Dedicated capacity based on the configuration
     * specified in the request.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be a minimum of 3 characters, and a maximum of 63.
     *
     * @param {object} capacityParameters Contains the information used to
     * provision the Dedicated capacity.
     *
     * @param {object} [capacityParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {string} capacityParameters.location Location of the PowerBI
     * Dedicated resource.
     *
     * @param {object} capacityParameters.sku The SKU of the PowerBI Dedicated
     * resource.
     *
     * @param {string} capacityParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityParameters.sku.tier] The name of the Azure pricing
     * tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityParameters.tags] Key-value pairs of additional
     * resource provisioning properties.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacity>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacity>>;

    /**
     * Provisions the specified Dedicated capacity based on the configuration
     * specified in the request.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be a minimum of 3 characters, and a maximum of 63.
     *
     * @param {object} capacityParameters Contains the information used to
     * provision the Dedicated capacity.
     *
     * @param {object} [capacityParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {string} capacityParameters.location Location of the PowerBI
     * Dedicated resource.
     *
     * @param {object} capacityParameters.sku The SKU of the PowerBI Dedicated
     * resource.
     *
     * @param {string} capacityParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityParameters.sku.tier] The name of the Azure pricing
     * tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityParameters.tags] Key-value pairs of additional
     * resource provisioning properties.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacity} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacity} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacity} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacity>;
    create(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, callback: ServiceCallback<models.DedicatedCapacity>): void;
    create(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacity>): void;


    /**
     * Deletes the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates the current state of the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} capacityUpdateParameters Request object that contains the
     * updated information for the capacity.
     *
     * @param {object} [capacityUpdateParameters.sku] The SKU of the Dedicated
     * capacity resource.
     *
     * @param {string} capacityUpdateParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityUpdateParameters.sku.tier] The name of the Azure
     * pricing tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityUpdateParameters.tags] Key-value pairs of
     * additional provisioning properties.
     *
     * @param {object} [capacityUpdateParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityUpdateParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacity>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacity>>;

    /**
     * Updates the current state of the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} capacityUpdateParameters Request object that contains the
     * updated information for the capacity.
     *
     * @param {object} [capacityUpdateParameters.sku] The SKU of the Dedicated
     * capacity resource.
     *
     * @param {string} capacityUpdateParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityUpdateParameters.sku.tier] The name of the Azure
     * pricing tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityUpdateParameters.tags] Key-value pairs of
     * additional provisioning properties.
     *
     * @param {object} [capacityUpdateParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityUpdateParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacity} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacity} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacity} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacity>;
    update(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, callback: ServiceCallback<models.DedicatedCapacity>): void;
    update(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacity>): void;


    /**
     * Suspends operation of the specified dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    suspendWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Suspends operation of the specified dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    suspend(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    suspend(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<void>): void;
    suspend(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Resumes operation of the specified Dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    resumeWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Resumes operation of the specified Dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    resume(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    resume(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<void>): void;
    resume(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Gets all the Dedicated capacities for the given resource group.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacities>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacities>>;

    /**
     * Gets all the Dedicated capacities for the given resource group.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacities} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacities} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacities} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacities>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.DedicatedCapacities>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacities>): void;


    /**
     * Lists all the Dedicated capacities for the given subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacities>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacities>>;

    /**
     * Lists all the Dedicated capacities for the given subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacities} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacities} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacities} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacities>;
    list(callback: ServiceCallback<models.DedicatedCapacities>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacities>): void;


    /**
     * Lists eligible SKUs for PowerBI Dedicated resource provider.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SkuEnumerationForNewResourceResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listSkusWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SkuEnumerationForNewResourceResult>>;

    /**
     * Lists eligible SKUs for PowerBI Dedicated resource provider.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SkuEnumerationForNewResourceResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SkuEnumerationForNewResourceResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SkuEnumerationForNewResourceResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listSkus(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SkuEnumerationForNewResourceResult>;
    listSkus(callback: ServiceCallback<models.SkuEnumerationForNewResourceResult>): void;
    listSkus(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SkuEnumerationForNewResourceResult>): void;


    /**
     * Lists eligible SKUs for a PowerBI Dedicated resource.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SkuEnumerationForExistingResourceResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listSkusForCapacityWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SkuEnumerationForExistingResourceResult>>;

    /**
     * Lists eligible SKUs for a PowerBI Dedicated resource.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SkuEnumerationForExistingResourceResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SkuEnumerationForExistingResourceResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SkuEnumerationForExistingResourceResult} for
     *                      more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listSkusForCapacity(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SkuEnumerationForExistingResourceResult>;
    listSkusForCapacity(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<models.SkuEnumerationForExistingResourceResult>): void;
    listSkusForCapacity(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SkuEnumerationForExistingResourceResult>): void;


    /**
     * Provisions the specified Dedicated capacity based on the configuration
     * specified in the request.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be a minimum of 3 characters, and a maximum of 63.
     *
     * @param {object} capacityParameters Contains the information used to
     * provision the Dedicated capacity.
     *
     * @param {object} [capacityParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {string} capacityParameters.location Location of the PowerBI
     * Dedicated resource.
     *
     * @param {object} capacityParameters.sku The SKU of the PowerBI Dedicated
     * resource.
     *
     * @param {string} capacityParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityParameters.sku.tier] The name of the Azure pricing
     * tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityParameters.tags] Key-value pairs of additional
     * resource provisioning properties.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacity>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacity>>;

    /**
     * Provisions the specified Dedicated capacity based on the configuration
     * specified in the request.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be a minimum of 3 characters, and a maximum of 63.
     *
     * @param {object} capacityParameters Contains the information used to
     * provision the Dedicated capacity.
     *
     * @param {object} [capacityParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {string} capacityParameters.location Location of the PowerBI
     * Dedicated resource.
     *
     * @param {object} capacityParameters.sku The SKU of the PowerBI Dedicated
     * resource.
     *
     * @param {string} capacityParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityParameters.sku.tier] The name of the Azure pricing
     * tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityParameters.tags] Key-value pairs of additional
     * resource provisioning properties.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacity} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacity} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacity} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacity>;
    beginCreate(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, callback: ServiceCallback<models.DedicatedCapacity>): void;
    beginCreate(resourceGroupName: string, dedicatedCapacityName: string, capacityParameters: models.DedicatedCapacity, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacity>): void;


    /**
     * Deletes the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates the current state of the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} capacityUpdateParameters Request object that contains the
     * updated information for the capacity.
     *
     * @param {object} [capacityUpdateParameters.sku] The SKU of the Dedicated
     * capacity resource.
     *
     * @param {string} capacityUpdateParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityUpdateParameters.sku.tier] The name of the Azure
     * pricing tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityUpdateParameters.tags] Key-value pairs of
     * additional provisioning properties.
     *
     * @param {object} [capacityUpdateParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityUpdateParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<DedicatedCapacity>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.DedicatedCapacity>>;

    /**
     * Updates the current state of the specified Dedicated capacity.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} capacityUpdateParameters Request object that contains the
     * updated information for the capacity.
     *
     * @param {object} [capacityUpdateParameters.sku] The SKU of the Dedicated
     * capacity resource.
     *
     * @param {string} capacityUpdateParameters.sku.name Name of the SKU level.
     *
     * @param {string} [capacityUpdateParameters.sku.tier] The name of the Azure
     * pricing tier to which the SKU applies. Possible values include: 'PBIE_Azure'
     *
     * @param {object} [capacityUpdateParameters.tags] Key-value pairs of
     * additional provisioning properties.
     *
     * @param {object} [capacityUpdateParameters.administration] A collection of
     * Dedicated capacity administrators
     *
     * @param {array} [capacityUpdateParameters.administration.members] An array of
     * administrator user identities.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {DedicatedCapacity} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {DedicatedCapacity} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link DedicatedCapacity} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.DedicatedCapacity>;
    beginUpdate(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, callback: ServiceCallback<models.DedicatedCapacity>): void;
    beginUpdate(resourceGroupName: string, dedicatedCapacityName: string, capacityUpdateParameters: models.DedicatedCapacityUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.DedicatedCapacity>): void;


    /**
     * Suspends operation of the specified dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginSuspendWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Suspends operation of the specified dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginSuspend(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginSuspend(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<void>): void;
    beginSuspend(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Resumes operation of the specified Dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginResumeWithHttpOperationResponse(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Resumes operation of the specified Dedicated capacity instance.
     *
     * @param {string} resourceGroupName The name of the Azure Resource group of
     * which a given PowerBIDedicated capacity is part. This name must be at least
     * 1 character in length, and no more than 90.
     *
     * @param {string} dedicatedCapacityName The name of the Dedicated capacity. It
     * must be at least 3 characters in length, and no more than 63.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginResume(resourceGroupName: string, dedicatedCapacityName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginResume(resourceGroupName: string, dedicatedCapacityName: string, callback: ServiceCallback<void>): void;
    beginResume(resourceGroupName: string, dedicatedCapacityName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the PowerBIDedicatedManagementClient.
 */
export interface Operations {


    /**
     * Lists all of the available PowerBIDedicated REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available PowerBIDedicated REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;


    /**
     * Lists all of the available PowerBIDedicated REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available PowerBIDedicated REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.OperationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}
