/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";
import * as moment from "moment";

export {

  BaseResource,
  CloudError
};

/**
 * SKU parameters supplied to the create Redis operation.
 */
export interface Sku {
  /**
   * The type of Redis cache to deploy. Valid values: (Basic, Standard, Premium). Possible values
   * include: 'Basic', 'Standard', 'Premium'
   */
  name: string;
  /**
   * The SKU family to use. Valid values: (C, P). (C = Basic/Standard, P = Premium). Possible
   * values include: 'C', 'P'
   */
  family: string;
  /**
   * The size of the Redis cache to deploy. Valid values: for C (Basic/Standard) family (0, 1, 2,
   * 3, 4, 5, 6), for P (Premium) family (1, 2, 3, 4).
   */
  capacity: number;
}

/**
 * Redis cache access keys.
 */
export interface RedisAccessKeys {
  /**
   * The current primary key that clients can use to authenticate with Redis cache.
   */
  readonly primaryKey?: string;
  /**
   * The current secondary key that clients can use to authenticate with Redis cache.
   */
  readonly secondaryKey?: string;
}

/**
 * Linked server Id
 */
export interface RedisLinkedServer {
  /**
   * Linked server Id.
   */
  readonly id?: string;
}

/**
 * The Resource definition.
 */
export interface Resource extends BaseResource {
  /**
   * Resource ID.
   */
  readonly id?: string;
  /**
   * Resource name.
   */
  readonly name?: string;
  /**
   * Resource type.
   */
  readonly type?: string;
}

/**
 * The resource model definition for a ARM proxy resource. It will have everything other than
 * required location and tags
 */
export interface ProxyResource extends Resource {
}

/**
 * The resource model definition for a ARM tracked top level resource
 */
export interface TrackedResource extends Resource {
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
  /**
   * The geo-location where the resource lives
   */
  location: string;
}

/**
 * Parameters supplied to the Create Redis operation.
 */
export interface RedisCreateParameters {
  /**
   * All Redis Settings. Few possible keys:
   * rdb-backup-enabled,rdb-storage-connection-string,rdb-backup-frequency,maxmemory-delta,maxmemory-policy,notify-keyspace-events,maxmemory-samples,slowlog-log-slower-than,slowlog-max-len,list-max-ziplist-entries,list-max-ziplist-value,hash-max-ziplist-entries,hash-max-ziplist-value,set-max-intset-entries,zset-max-ziplist-entries,zset-max-ziplist-value
   * etc.
   */
  redisConfiguration?: { [propertyName: string]: string };
  /**
   * Specifies whether the non-ssl Redis server port (6379) is enabled.
   */
  enableNonSslPort?: boolean;
  /**
   * A dictionary of tenant settings
   */
  tenantSettings?: { [propertyName: string]: string };
  /**
   * The number of shards to be created on a Premium Cluster Cache.
   */
  shardCount?: number;
  /**
   * Optional: requires clients to use a specified TLS version (or higher) to connect (e,g, '1.0',
   * '1.1', '1.2'). Possible values include: '1.0', '1.1', '1.2'
   */
  minimumTlsVersion?: string;
  /**
   * The SKU of the Redis cache to deploy.
   */
  sku: Sku;
  /**
   * The full resource ID of a subnet in a virtual network to deploy the Redis cache in. Example
   * format:
   * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/Microsoft.{Network|ClassicNetwork}/VirtualNetworks/vnet1/subnets/subnet1
   */
  subnetId?: string;
  /**
   * Static IP address. Required when deploying a Redis cache inside an existing Azure Virtual
   * Network.
   */
  staticIP?: string;
  /**
   * A list of availability zones denoting where the resource needs to come from.
   */
  zones?: string[];
  /**
   * The geo-location where the resource lives
   */
  location: string;
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * Parameters supplied to the Update Redis operation.
 */
export interface RedisUpdateParameters {
  /**
   * All Redis Settings. Few possible keys:
   * rdb-backup-enabled,rdb-storage-connection-string,rdb-backup-frequency,maxmemory-delta,maxmemory-policy,notify-keyspace-events,maxmemory-samples,slowlog-log-slower-than,slowlog-max-len,list-max-ziplist-entries,list-max-ziplist-value,hash-max-ziplist-entries,hash-max-ziplist-value,set-max-intset-entries,zset-max-ziplist-entries,zset-max-ziplist-value
   * etc.
   */
  redisConfiguration?: { [propertyName: string]: string };
  /**
   * Specifies whether the non-ssl Redis server port (6379) is enabled.
   */
  enableNonSslPort?: boolean;
  /**
   * A dictionary of tenant settings
   */
  tenantSettings?: { [propertyName: string]: string };
  /**
   * The number of shards to be created on a Premium Cluster Cache.
   */
  shardCount?: number;
  /**
   * Optional: requires clients to use a specified TLS version (or higher) to connect (e,g, '1.0',
   * '1.1', '1.2'). Possible values include: '1.0', '1.1', '1.2'
   */
  minimumTlsVersion?: string;
  /**
   * The SKU of the Redis cache to deploy.
   */
  sku?: Sku;
  /**
   * Resource tags.
   */
  tags?: { [propertyName: string]: string };
}

/**
 * Specifies a range of IP addresses permitted to connect to the cache
 */
export interface RedisFirewallRuleProperties {
  /**
   * lowest IP address included in the range
   */
  startIP: string;
  /**
   * highest IP address included in the range
   */
  endIP: string;
}

/**
 * A firewall rule on a redis cache has a name, and describes a contiguous range of IP addresses
 * permitted to connect
 */
export interface RedisFirewallRule extends ProxyResource {
  /**
   * lowest IP address included in the range
   */
  startIP: string;
  /**
   * highest IP address included in the range
   */
  endIP: string;
}

/**
 * Parameters required for creating a firewall rule on redis cache.
 */
export interface RedisFirewallRuleCreateParameters {
  /**
   * lowest IP address included in the range
   */
  startIP: string;
  /**
   * highest IP address included in the range
   */
  endIP: string;
}

/**
 * A single Redis item in List or Get Operation.
 */
export interface RedisResource extends TrackedResource {
  /**
   * All Redis Settings. Few possible keys:
   * rdb-backup-enabled,rdb-storage-connection-string,rdb-backup-frequency,maxmemory-delta,maxmemory-policy,notify-keyspace-events,maxmemory-samples,slowlog-log-slower-than,slowlog-max-len,list-max-ziplist-entries,list-max-ziplist-value,hash-max-ziplist-entries,hash-max-ziplist-value,set-max-intset-entries,zset-max-ziplist-entries,zset-max-ziplist-value
   * etc.
   */
  redisConfiguration?: { [propertyName: string]: string };
  /**
   * Specifies whether the non-ssl Redis server port (6379) is enabled.
   */
  enableNonSslPort?: boolean;
  /**
   * A dictionary of tenant settings
   */
  tenantSettings?: { [propertyName: string]: string };
  /**
   * The number of shards to be created on a Premium Cluster Cache.
   */
  shardCount?: number;
  /**
   * Optional: requires clients to use a specified TLS version (or higher) to connect (e,g, '1.0',
   * '1.1', '1.2'). Possible values include: '1.0', '1.1', '1.2'
   */
  minimumTlsVersion?: string;
  /**
   * The SKU of the Redis cache to deploy.
   */
  sku: Sku;
  /**
   * The full resource ID of a subnet in a virtual network to deploy the Redis cache in. Example
   * format:
   * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/Microsoft.{Network|ClassicNetwork}/VirtualNetworks/vnet1/subnets/subnet1
   */
  subnetId?: string;
  /**
   * Static IP address. Required when deploying a Redis cache inside an existing Azure Virtual
   * Network.
   */
  staticIP?: string;
  /**
   * Redis version.
   */
  readonly redisVersion?: string;
  /**
   * Redis instance provisioning status. Possible values include: 'Creating', 'Deleting',
   * 'Disabled', 'Failed', 'Linking', 'Provisioning', 'RecoveringScaleFailure', 'Scaling',
   * 'Succeeded', 'Unlinking', 'Unprovisioning', 'Updating'
   */
  readonly provisioningState?: string;
  /**
   * Redis host name.
   */
  readonly hostName?: string;
  /**
   * Redis non-SSL port.
   */
  readonly port?: number;
  /**
   * Redis SSL port.
   */
  readonly sslPort?: number;
  /**
   * The keys of the Redis cache - not set if this object is not the response to Create or Update
   * redis cache
   */
  readonly accessKeys?: RedisAccessKeys;
  /**
   * List of the linked servers associated with the cache
   */
  readonly linkedServers?: RedisLinkedServer[];
  /**
   * A list of availability zones denoting where the resource needs to come from.
   */
  zones?: string[];
}

/**
 * Specifies which Redis access keys to reset.
 */
export interface RedisRegenerateKeyParameters {
  /**
   * The Redis access key to regenerate. Possible values include: 'Primary', 'Secondary'
   */
  keyType: string;
}

/**
 * Specifies which Redis node(s) to reboot.
 */
export interface RedisRebootParameters {
  /**
   * Which Redis node(s) to reboot. Depending on this value data loss is possible. Possible values
   * include: 'PrimaryNode', 'SecondaryNode', 'AllNodes'
   */
  rebootType: string;
  /**
   * If clustering is enabled, the ID of the shard to be rebooted.
   */
  shardId?: number;
}

/**
 * Parameters for Redis export operation.
 */
export interface ExportRDBParameters {
  /**
   * File format.
   */
  format?: string;
  /**
   * Prefix to use for exported files.
   */
  prefix: string;
  /**
   * Container name to export to.
   */
  container: string;
}

/**
 * Parameters for Redis import operation.
 */
export interface ImportRDBParameters {
  /**
   * File format.
   */
  format?: string;
  /**
   * files to import.
   */
  files: string[];
}

/**
 * Patch schedule entry for a Premium Redis Cache.
 */
export interface ScheduleEntry {
  /**
   * Day of the week when a cache can be patched. Possible values include: 'Monday', 'Tuesday',
   * 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday', 'Everyday', 'Weekend'
   */
  dayOfWeek: string;
  /**
   * Start hour after which cache patching can start.
   */
  startHourUtc: number;
  /**
   * ISO8601 timespan specifying how much time cache patching can take.
   */
  maintenanceWindow?: moment.Duration;
}

/**
 * List of patch schedules for a Redis cache.
 */
export interface ScheduleEntries {
  /**
   * List of patch schedules for a Redis cache.
   */
  scheduleEntries: ScheduleEntry[];
}

/**
 * Response to put/get patch schedules for Redis cache.
 */
export interface RedisPatchSchedule extends ProxyResource {
  /**
   * List of patch schedules for a Redis cache.
   */
  scheduleEntries: ScheduleEntry[];
}

/**
 * Response to force reboot for Redis cache.
 */
export interface RedisForceRebootResponse {
  /**
   * Status message
   */
  readonly message?: string;
}

/**
 * Create properties for a linked server
 */
export interface RedisLinkedServerCreateProperties {
  /**
   * Fully qualified resourceId of the linked redis cache.
   */
  linkedRedisCacheId: string;
  /**
   * Location of the linked redis cache.
   */
  linkedRedisCacheLocation: string;
  /**
   * Role of the linked server. Possible values include: 'Primary', 'Secondary'
   */
  serverRole: string;
}

/**
 * Properties of a linked server to be returned in get/put response
 */
export interface RedisLinkedServerProperties extends RedisLinkedServerCreateProperties {
  /**
   * Terminal state of the link between primary and secondary redis cache.
   */
  readonly provisioningState?: string;
}

/**
 * Response to put/get linked server (with properties) for Redis cache.
 */
export interface RedisLinkedServerWithProperties extends ProxyResource {
  /**
   * Fully qualified resourceId of the linked redis cache.
   */
  linkedRedisCacheId: string;
  /**
   * Location of the linked redis cache.
   */
  linkedRedisCacheLocation: string;
  /**
   * Role of the linked server. Possible values include: 'Primary', 'Secondary'
   */
  serverRole: string;
  /**
   * Terminal state of the link between primary and secondary redis cache.
   */
  readonly provisioningState?: string;
}

/**
 * Parameter required for creating a linked server to redis cache.
 */
export interface RedisLinkedServerCreateParameters {
  /**
   * Fully qualified resourceId of the linked redis cache.
   */
  linkedRedisCacheId: string;
  /**
   * Location of the linked redis cache.
   */
  linkedRedisCacheLocation: string;
  /**
   * Role of the linked server. Possible values include: 'Primary', 'Secondary'
   */
  serverRole: string;
}

/**
 * The object that describes the operation.
 */
export interface OperationDisplay {
  /**
   * Friendly name of the resource provider
   */
  provider?: string;
  /**
   * Operation type: read, write, delete, listKeys/action, etc.
   */
  operation?: string;
  /**
   * Resource type on which the operation is performed.
   */
  resource?: string;
  /**
   * Friendly name of the operation
   */
  description?: string;
}

/**
 * REST API operation
 */
export interface Operation {
  /**
   * Operation name: {provider}/{resource}/{operation}
   */
  name?: string;
  /**
   * The object that describes the operation.
   */
  display?: OperationDisplay;
}

/**
 * Parameters body to pass for resource name availability check.
 */
export interface CheckNameAvailabilityParameters {
  /**
   * Resource name.
   */
  name: string;
  /**
   * Resource type. The only legal value of this property for checking redis cache name
   * availability is 'Microsoft.Cache/redis'.
   */
  type: string;
}

/**
 * Properties of upgrade notification.
 */
export interface UpgradeNotification {
  /**
   * Name of upgrade notification.
   */
  readonly name?: string;
  /**
   * Timestamp when upgrade notification occurred.
   */
  readonly timestamp?: Date;
  /**
   * Details about this upgrade notification
   */
  readonly upsellNotification?: { [propertyName: string]: string };
}

/**
 * The response of listUpgradeNotifications.
 */
export interface NotificationListResponse {
  /**
   * List of all notifications.
   */
  value?: UpgradeNotification[];
  /**
   * Link for next set of notifications.
   */
  readonly nextLink?: string;
}

/**
 * Result of the request to list REST API operations. It contains a list of operations and a URL
 * nextLink to get the next set of results.
 */
export interface OperationListResult extends Array<Operation> {
  /**
   * URL to get the next set of operation list results if there are any.
   */
  readonly nextLink?: string;
}

/**
 * The response of list Redis operation.
 */
export interface RedisListResult extends Array<RedisResource> {
  /**
   * Link for next page of results.
   */
  readonly nextLink?: string;
}

/**
 * The response of list firewall rules Redis operation.
 */
export interface RedisFirewallRuleListResult extends Array<RedisFirewallRule> {
  /**
   * Link for next page of results.
   */
  readonly nextLink?: string;
}

/**
 * The response of list patch schedules Redis operation.
 */
export interface RedisPatchScheduleListResult extends Array<RedisPatchSchedule> {
  /**
   * Link for next page of results.
   */
  readonly nextLink?: string;
}

/**
 * List of linked servers (with properties) of a Redis cache.
 */
export interface RedisLinkedServerWithPropertiesList extends Array<RedisLinkedServerWithProperties> {
  /**
   * Link for next set.
   */
  readonly nextLink?: string;
}
