/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for applications, containing health evaluations
 * for each unhealthy application that impacted current aggregated health
 * state.
 *
 * @extends models['HealthEvaluation']
 */
class ApplicationsHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a ApplicationsHealthEvaluation.
   * @property {number} [maxPercentUnhealthyApplications] Maximum allowed
   * percentage of unhealthy applications from the ClusterHealthPolicy.
   * @property {number} [totalCount] Total number of applications from the
   * health store.
   * @property {array} [unhealthyEvaluations] List of unhealthy evaluations
   * that led to the aggregated health state. Includes all the unhealthy
   * ApplicationHealthEvaluation that impacted the aggregated health.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ApplicationsHealthEvaluation
   *
   * @returns {object} metadata of ApplicationsHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Applications',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'Kind',
          clientName: 'kind'
        },
        uberParent: 'HealthEvaluation',
        className: 'ApplicationsHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          maxPercentUnhealthyApplications: {
            required: false,
            serializedName: 'MaxPercentUnhealthyApplications',
            type: {
              name: 'Number'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ApplicationsHealthEvaluation;
