/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for a service, containing information about the
 * data and the algorithm used by health store to evaluate health. The
 * evaluation is returned only when the aggregated health state is either Error
 * or Warning.
 *
 * @extends models['HealthEvaluation']
 */
class ServiceHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a ServiceHealthEvaluation.
   * @property {string} [serviceName] Name of the service whose health
   * evaluation is described by this object.
   * @property {array} [unhealthyEvaluations] List of unhealthy evaluations
   * that led to the current aggregated health state of the service. The types
   * of the unhealthy evaluations can be PartitionsHealthEvaluation or
   * EventHealthEvaluation.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ServiceHealthEvaluation
   *
   * @returns {object} metadata of ServiceHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Service',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'Kind',
          clientName: 'kind'
        },
        uberParent: 'HealthEvaluation',
        className: 'ServiceHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          serviceName: {
            required: false,
            serializedName: 'ServiceName',
            type: {
              name: 'String'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ServiceHealthEvaluation;
