/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents a stateful service replica. This includes information about the
 * identity, role, status, health, node name, uptime, and other details about
 * the replica.
 *
 * @extends models['ReplicaInfo']
 */
class StatefulServiceReplicaInfo extends models['ReplicaInfo'] {
  /**
   * Create a StatefulServiceReplicaInfo.
   * @property {string} [replicaRole] The role of a replica of a stateful
   * service. Possible values include: 'Unknown', 'None', 'Primary',
   * 'IdleSecondary', 'ActiveSecondary'
   * @property {string} [replicaId] Id of a stateful service replica. ReplicaId
   * is used by Service Fabric to uniquely identify a replica of a partition.
   * It is unique within a partition and does not change for the lifetime of
   * the replica. If a replica gets dropped and another replica gets created on
   * the same node for the same partition, it will get a different value for
   * the id. Sometimes the id of a stateless service instance is also referred
   * as a replica id.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of StatefulServiceReplicaInfo
   *
   * @returns {object} metadata of StatefulServiceReplicaInfo
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Stateful',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'ServiceKind',
          clientName: 'serviceKind'
        },
        uberParent: 'ReplicaInfo',
        className: 'StatefulServiceReplicaInfo',
        modelProperties: {
          replicaStatus: {
            required: false,
            serializedName: 'ReplicaStatus',
            type: {
              name: 'String'
            }
          },
          healthState: {
            required: false,
            serializedName: 'HealthState',
            type: {
              name: 'String'
            }
          },
          nodeName: {
            required: false,
            serializedName: 'NodeName',
            type: {
              name: 'String'
            }
          },
          address: {
            required: false,
            serializedName: 'Address',
            type: {
              name: 'String'
            }
          },
          lastInBuildDurationInSeconds: {
            required: false,
            serializedName: 'LastInBuildDurationInSeconds',
            type: {
              name: 'String'
            }
          },
          serviceKind: {
            required: true,
            serializedName: 'ServiceKind',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          replicaRole: {
            required: false,
            serializedName: 'ReplicaRole',
            type: {
              name: 'String'
            }
          },
          replicaId: {
            required: false,
            serializedName: 'ReplicaId',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = StatefulServiceReplicaInfo;
