/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Clone Job Request Model.
 *
 */
class CloneRequest {
  /**
   * Create a CloneRequest.
   * @member {string} targetDeviceId DeviceId of the device which will act as
   * the Clone target
   * @member {string} targetAccessPointId Accesspoint Id on which clone job
   * will performed.
   * @member {string} newEndpointName Name of new endpoint which will created
   * as part of clone job.
   * @member {object} [share] Share Object.
   * @member {string} [share.description] Description for file share
   * @member {string} [share.shareStatus] The Share Status. Possible values
   * include: 'Online', 'Offline'
   * @member {string} [share.dataPolicy] The data policy. Possible values
   * include: 'Invalid', 'Local', 'Tiered', 'Cloud'
   * @member {string} [share.adminUser] The user/group who will have full
   * permission in this share. Active directory email address. Example:
   * xyz@contoso.com or Contoso\xyz.
   * @member {number} [share.provisionedCapacityInBytes] The total provisioned
   * capacity in Bytes
   * @member {number} [share.usedCapacityInBytes] The used capacity in Bytes.
   * @member {number} [share.localUsedCapacityInBytes] The local used capacity
   * in Bytes.
   * @member {string} [share.monitoringStatus] The monitoring status. Possible
   * values include: 'Enabled', 'Disabled'
   * @member {object} [disk] Disk Object
   * @member {string} [disk.description] The description.
   * @member {string} [disk.diskStatus] The disk status. Possible values
   * include: 'Online', 'Offline'
   * @member {array} [disk.accessControlRecords] The access control records.
   * @member {string} [disk.dataPolicy] The data policy. Possible values
   * include: 'Invalid', 'Local', 'Tiered', 'Cloud'
   * @member {number} [disk.provisionedCapacityInBytes] The provisioned
   * capacity in bytes.
   * @member {number} [disk.usedCapacityInBytes] The used capacity in bytes.
   * @member {number} [disk.localUsedCapacityInBytes] The local used capacity
   * in bytes.
   * @member {string} [disk.monitoringStatus] The monitoring. Possible values
   * include: 'Enabled', 'Disabled'
   */
  constructor() {
  }

  /**
   * Defines the metadata of CloneRequest
   *
   * @returns {object} metadata of CloneRequest
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CloneRequest',
      type: {
        name: 'Composite',
        className: 'CloneRequest',
        modelProperties: {
          targetDeviceId: {
            required: true,
            serializedName: 'properties.targetDeviceId',
            type: {
              name: 'String'
            }
          },
          targetAccessPointId: {
            required: true,
            serializedName: 'properties.targetAccessPointId',
            type: {
              name: 'String'
            }
          },
          newEndpointName: {
            required: true,
            serializedName: 'properties.newEndpointName',
            type: {
              name: 'String'
            }
          },
          share: {
            required: false,
            serializedName: 'properties.share',
            type: {
              name: 'Composite',
              className: 'FileShare'
            }
          },
          disk: {
            required: false,
            serializedName: 'properties.disk',
            type: {
              name: 'Composite',
              className: 'ISCSIDisk'
            }
          }
        }
      }
    };
  }
}

module.exports = CloneRequest;
