/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Parameters supplied to the Update Event Source operation to update an
 * EventHub event source.
 *
 * @extends models['EventSourceUpdateParameters']
 */
class EventHubEventSourceUpdateParameters extends models['EventSourceUpdateParameters'] {
  /**
   * Create a EventHubEventSourceUpdateParameters.
   * @member {string} [timestampPropertyName] The event property that will be
   * used as the event source's timestamp. If a value isn't specified for
   * timestampPropertyName, or if null or empty-string is specified, the event
   * creation time will be used.
   * @member {object} [localTimestamp] An object that represents the local
   * timestamp property. It contains the format of local timestamp that needs
   * to be used and the corresponding timezone offset information. If a value
   * isn't specified for localTimestamp, or if null, then the local timestamp
   * will not be ingressed with the events.
   * @member {string} [localTimestamp.format] An enum that represents the
   * format of the local timestamp property that needs to be set. Possible
   * values include: 'Embedded', 'Iana', 'TimeSpan'
   * @member {object} [localTimestamp.timeZoneOffset] An object that represents
   * the offset information for the local timestamp format specified. Should
   * not be specified for LocalTimestampFormat - Embedded.
   * @member {string} [localTimestamp.timeZoneOffset.propertyName] The event
   * property that will be contain the offset information to calculate the
   * local timestamp. When the LocalTimestampFormat is Iana, the property name
   * will contain the name of the column which contains IANA Timezone Name (eg:
   * Americas/Los Angeles). When LocalTimestampFormat is Timespan, it contains
   * the name of property which contains values representing the offset (eg:
   * P1D or 1.00:00:00)
   * @member {string} [sharedAccessKey] The value of the shared access key that
   * grants the Time Series Insights service read access to the event hub. This
   * property is not shown in event source responses.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of EventHubEventSourceUpdateParameters
   *
   * @returns {object} metadata of EventHubEventSourceUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'EventHubEventSourceUpdateParameters',
      type: {
        name: 'Composite',
        className: 'EventHubEventSourceUpdateParameters',
        modelProperties: {
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          timestampPropertyName: {
            required: false,
            serializedName: 'properties.timestampPropertyName',
            type: {
              name: 'String'
            }
          },
          localTimestamp: {
            required: false,
            serializedName: 'properties.localTimestamp',
            type: {
              name: 'Composite',
              className: 'LocalTimestamp'
            }
          },
          sharedAccessKey: {
            required: false,
            serializedName: 'properties.sharedAccessKey',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = EventHubEventSourceUpdateParameters;
